#include "StdAfx.h"

#include ".\blogtooltype.h"
#include "feedManager.h"
#include "FeedManagerErrorCode.h"
#include "ExceptionBase.h"
#include "FeedManagerLibHelper.h"
#include "GNUtil.h"

CBlogToolType::CBlogToolType(void)
{
	m_type = Unknown;
	m_hPlugin = NULL;
	m_pGetNameProc = NULL;
	m_pBlogThisProc = NULL;
}

CBlogToolType::CBlogToolType(LPCTSTR name, LPCTSTR url, Type type)
{
	m_name = name;
	m_defaultURL = url;
	m_type = type;

	m_hPlugin = NULL;
	m_pGetNameProc = NULL;
	m_pBlogThisProc = NULL;
}

CBlogToolType::CBlogToolType(LPCTSTR pluginPath)
{
	m_type = Unknown;
	m_hPlugin = NULL;
	m_pGetNameProc = NULL;
	m_pBlogThisProc = NULL;

	try
	{
		m_hPlugin = ::LoadLibrary(pluginPath);
		if(m_hPlugin == NULL)
			return;

		m_pGetNameProc = (GET_NAME_PROC)::GetProcAddress(m_hPlugin, "GetGreatNewsPluginName");
		m_pBlogThisProc = (BLOG_THIS_PROC)::GetProcAddress(m_hPlugin, "BlogThis");
		
		m_name = m_pGetNameProc();

		m_type = Plugin; // everything is all set, let's set the type
	}
	catch(...)
	{
	}
}

CBlogToolType::~CBlogToolType(void)
{
	if(m_hPlugin)
		::FreeLibrary(m_hPlugin);
}

size_t CBlogToolType::GetAllBlogToolTyes(BlogToolTypeVector& allTypes, const CString& pluginPath)
{
	allTypes.clear();
	allTypes.reserve(20);

	//
	// load all buildin blog tools
	//
	CppSQLite3DB db;
	CFeedManager::OpenDatabase(db);

	try
	{
		CString sql = _T("select tool_type_name, default_url from blog_tool_type;");
		CppSQLite3Query q = db.execQuery(sql);
		while(!q.eof())
		{
			allTypes.push_back(new CBlogToolType(q.getStringField(0),q.getStringField(1)));
			q.nextRow();
		}
		q.finalize();
		db.close();
	}
	catch(CppSQLite3Exception& e)
	{
		throw CExceptionBase(ERR_FM_DBERROR, e.errorMessage());
	}

	//allTypes.push_back(new CBlogToolType(_T("Drupal"), _T("http://yoursite.com/drupal/")));
	//allTypes.push_back(new CBlogToolType(_T("Movable Type"), _T("http://yoursite.com/mt/mt.cgi")));
	//allTypes.push_back(new CBlogToolType(_T("WordPress"), _T("http://yoursite.com/wordpress/wp-admin/bookmarklet.php")));
	//allTypes.push_back(new CBlogToolType(_T("Generic"), _T("http://yoursite.com/post?title=%TITLE%&text=%TEXT%&url=%URL%")));

	// Add this special one
	allTypes.push_back(new CBlogToolGenericExternalType());


	//
	// Load all plugins
	//
	CFindFile finder;
	BOOL bFound = finder.FindFile(pluginPath + _T("\\*.dll"));
	while(bFound)
	{
		if(IsGreatNewsPlugin(finder.GetFilePath()))
			allTypes.push_back(new CBlogToolType(finder.GetFilePath()));
		bFound = finder.FindNextFile();
	}

	return allTypes.size();
}

bool CBlogToolType::IsGreatNewsPlugin(LPCTSTR filePath)
{
	try
	{
		HMODULE hInst = ::LoadLibrary(filePath);
		if(hInst == NULL)
			return false;

		FARPROC procName = ::GetProcAddress(hInst, "GetGreatNewsPluginName");
		FARPROC procBlogThis = ::GetProcAddress(hInst, "BlogThis");

		bool bValid = (procName != NULL && procBlogThis != NULL);

		::FreeLibrary(hInst);

		return bValid;

	}
	catch(...)
	{
		return false;
	}

	return false;
}

void CBlogToolType::BlogThis(NewsItemPtr& item, LPCTSTR pluginOptions)
{
	ATLASSERT(m_pBlogThisProc);

	MSXML2::IXMLDOMDocumentPtr spDoc = item->GenerateFeedXml();
	
	CString xml = (LPCTSTR)spDoc->xml;

//		xml = _T("\
//<rss version=\"0.91\">\
//  <channel>\
//    <title>Sam Ruby</title>\
//    <link>http://www.intertwingly.net/blog/</link>\
//    <item>\
//      <title>xhtml:body in Radio</title>\
//      <link>http://www.intertwingly.net/blog/1333.html</link>\
//      <description>&lt;p&gt;\
//&lt;a href=\"http://weblog.infoworld.com/udell/2003/04/14.html#a666\"&gt;Jon\
//Udell&lt;/a&gt;: &lt;em&gt;Sam Ruby and Don Box have both demonstrated valid\
//RSS 2.0 feeds that include a &amp;lt;body&amp;gt; element, properly\
//namespaced as XHTML. Quietly, last week, I joined the\
//party.&lt;/em&gt;&lt;/p&gt;\
//&lt;p&gt;Way to go Jon!&amp;nbsp; I'm very much looking forward to your\
//upcoming O'Reilly Network column.&lt;/p&gt;\
//</description>\
//    </item>\
//  </channel>\
//</rss>");

	m_pBlogThisProc(xml, pluginOptions);
}

//bool operator == (BlogToolTypePtr& type, LPCTSTR name)
//{
//	return (type->m_name == name);
//};


/////////////////////////////////////////////////////////////////////////////////////
//
//
CBlogToolGenericExternalType::CBlogToolGenericExternalType()
: CBlogToolType(_T("Generic external blog tool")
                ,_T("<Executable path to the tool and command line parameters>")
				,GenericExternal)
{
}

void CBlogToolGenericExternalType::BlogThis(NewsItemPtr& item, LPCTSTR pluginOptions)
{
	CString cmdLine = pluginOptions;
	cmdLine.Replace(_T("%TITLE%"), item->GetTitle());
	cmdLine.Replace(_T("%AUTHOR%"), item->m_author); 
	cmdLine.Replace(_T("%LINK%"), item->GetLink()); 
	cmdLine.Replace(_T("%TEXT%"), item->m_description); 

	if(cmdLine.Find(_T("%HTMLFILE%"))>0)
	{
		CString htmlContent;
		htmlContent.Format(_T("\
<blockquote>\
<a href='%s'>%s</a><br>\
%s\
</blockquote>"), (LPCTSTR)item->GetLink(), (LPCTSTR)item->GetTitle(), 
		(LPCTSTR)item->m_description);

	    TCHAR szTempFileName[MAX_PATH];
        ::GetTempPath(MAX_PATH, szTempFileName);
        ::GetTempFileName(szTempFileName, _T("w"), 0, szTempFileName);
		CString fileName = szTempFileName;
		CGNUtil::WriteToFile(fileName, htmlContent);
		cmdLine.Replace(_T("%HTMLFILE%"), fileName); 
	}

	STARTUPINFO si;
	memset(&si, 0, sizeof(STARTUPINFO));
	si.cb = sizeof(STARTUPINFO);
	PROCESS_INFORMATION pi;
	::CreateProcess(NULL, (LPTSTR)(LPCTSTR)cmdLine, NULL, NULL, false, CREATE_DEFAULT_ERROR_MODE, NULL, NULL, &si, &pi);

}