#include "StdAfx.h"
#include ".\bloglinesservice.h"

#include "CppSQLite3.h"
#include "ExceptionBase.h"
#include "FeedManagerErrorCode.h"
#include "GNUtil.h"
#include "FeedManager.h"
#include "FeedManagerLibHelper.h"

CBloglinesService::CBloglinesService(void) : m_dontMarkRead(false)
{
}

CBloglinesService::~CBloglinesService(void)
{
}

MSXML2::IXMLDOMDocumentPtr CBloglinesService::ListChannels(LPCTSTR account, LPCTSTR password)
{
	MSXML2::IXMLDOMDocumentPtr spChannelDoc;

	try
	{
		MSXML2::IXMLHTTPRequestPtr spRequest(_T("MSXML2.XMLHTTP.3.0"));

		spRequest->open(_T("GET"), _T("http://rpc.bloglines.com/listsubs"), false, account, password);
		spRequest->send();

		if(spRequest->status == 401)
		{
			CString msg;
			msg.Format(_T("Incorrect email address or password."));
			throw CExceptionBase(ERR_FM_GENERICERR, msg);
		}
		else if(spRequest->status != 200)
		{
			CString msg;
			msg.Format(_T("Error occured while accessing bloglines.com.[%d]"), spRequest->status);
			throw CExceptionBase(ERR_FM_GENERICERR, msg);
		}

		spChannelDoc.CreateInstance("MSXML2.DOMDocument.3.0");
		spChannelDoc->validateOnParse = false; // do not validate DTD/Schema
		spChannelDoc->resolveExternals = false;
		spChannelDoc->load(spRequest->responseStream);
		CGNUtil::CheckDomError(spChannelDoc);
	}
	catch(CExceptionBase& e)
	{
		spChannelDoc = NULL;
		m_errMsg = e.GetErrorMsg();
	}
	catch(...)
	{
		spChannelDoc = NULL;
		m_errMsg = _T("Unknown error occured while downloading bloglines.com subscriptions");
	}

	return spChannelDoc;
}

bool CBloglinesService::GetMarkReadFlag()
{
	if(!LoadParameters())
		return false;

	return m_dontMarkRead;
}

void CBloglinesService::SetMarkReadFlag(bool dontMark)
{
	m_dontMarkRead = dontMark;
}

bool CBloglinesService::GetAccountInfo(CString& account, CString& password)
{
	if(!LoadParameters())
		return false;

	account = m_account;
	password = m_password;

	return true;
}

bool CBloglinesService::SetAccountInfo(const CString& account, const CString& password)
{
	m_account = account;
	m_password = password;
	
	return true;
}

bool CBloglinesService::SaveParam()
{
	//int nAccountLen = Base64EncodeGetRequiredLength(account.GetLength()*sizeof(TCHAR));
	//CString account64;
	//Base64Encode((const BYTE*)(LPCTSTR)sSource, sSource.GetLength()*sizeof(TCHAR), 
	//			account64.GetBuffer(nAccountLen), &nAccountLen);
	//account64.ReleaseBuffer(nDestLen);

	//int nPasswordLen = Base64EncodeGetRequiredLength(password.GetLength()*sizeof(TCHAR));
	//CString password64;
	//Base64Encode((const BYTE*)(LPCTSTR)password, password.GetLength()*sizeof(TCHAR), 
	//			password64.GetBuffer(nPasswordLen), &nPasswordLen);
	//password64.ReleaseBuffer(nDestLen);

	CString sql;
	sql.Format(_T("update configurations set options7='%s', options8='%s', options6=%d"),
				(LPCTSTR)m_account, (LPCTSTR)m_password, (int)m_dontMarkRead);

	CFeedManagerLibHelper::DBExec(sql, CFeedManagerLibHelper::UseTransaction);

	return true;
}


CString CBloglinesService::GetSyncChannelUrl(int subsId)
{
	CString url;
	url.Format(_T("http://rpc.bloglines.com/getitems?s=%d&n=%d")
		,subsId
		,(m_dontMarkRead ? 0 : 1));
	return url;
}

bool CBloglinesService::LoadParameters()
{
	if(m_account.GetLength() == 0)
	{
		try
		{
			//
			// get from db
			//
			CppSQLite3DB db;
			CFeedManager::OpenDatabase(db);

			CppSQLite3Query q = db.execQuery(_T("select options7,options8,options6 from configurations;"));
			if(!q.eof())
			{
				m_account = q.getStringField(0,_T(""));
				m_password = q.getStringField(1,_T(""));
				m_dontMarkRead = q.getIntField(2,0);
			}
			q.finalize();
			db.close();
		}
		catch(CppSQLite3Exception&)
		{
			return false;
		}
	}

	return true;

}
