#include "StdAfx.h"

#include "FeedManagerLib.h"
#include "FeedManager.h"
#include "FeedManagerLibHelper.h"

#include "Singleton.h"

namespace FeedManagerLib
{

void Init(LPCTSTR dbFileName, bool bFullInit)
{
	//CppSQLite3DB::Init();

	CFeedManager::Init(dbFileName);
	// populate search channel definitions;
	// SearchChannelDefVector& defs = CSearchChannelDef::m_searchChannelDefs;
	//defs.push_back(	CSearchChannelDef(_T("Yahoo! - Company News via RSS"), _T("http://finance.yahoo.com/rss/headline?s=%KEYWORD%")));
	//defs.push_back(	CSearchChannelDef(_T("Feedster"), _T("http://www.feedster.com/search.php?q=%KEYWORD%&sort=date&content=full&type=rss&limit=20")));
	//defs.push_back(	CSearchChannelDef(_T("MSN"), _T("http://search.msn.com/results.aspx?q=%KEYWORD%&format=rss")));
	//defs.push_back(	CSearchChannelDef(_T("Flickr"), _T("http://www.flickr.com/services/feeds/photos_public.gne?tags=%KEYWORD%&format=rss_200")));
	//defs.push_back(	CSearchChannelDef(_T("Google Blog Search"), _T("http://blogsearch.google.com/blogsearch_feeds?q=%KEYWORD%&btnG=Search+Blogs&num=50&output=rss")));


	if(bFullInit)
	{
	}
}

void CompactDatabase()
{
	// vacuum cannot run within a transaction!
	CFeedManagerLibHelper::DBExec(_T("vacuum;"), CFeedManagerLibHelper::NoTransaction);
	CFeedManagerLibHelper::DBExec(_T("ANALYZE;"), CFeedManagerLibHelper::NoTransaction);
}

CString GetDbVersion()
{
	CString version;

	try
	{
		CppSQLite3DB db;
		CFeedManager::OpenDatabase(db);

		CppSQLite3Query q = db.execQuery(_T("select db_ver from configurations;"));
		if(!q.eof())
		{
			version.Format(_T("1.0.0.%d"), q.getIntField(0));
		}
		q.finalize();
		db.close();
	}
	catch(CppSQLite3Exception&)
	{
	}

	return version;
}

bool CheckLastShutdown()
{
	CppSQLite3DB db;
	CFeedManager::OpenDatabase(db);

	int lastStart, lastShutdown;
	lastStart = lastShutdown = 0;

	try
	{
		CppSQLite3Query q = db.execQuery(_T("select options1, options2 from configurations"));
		if(!q.eof())
		{
			lastStart = q.getIntField(0);
			lastShutdown = q.getIntField(1);
		}
		q.finalize();
	}
	catch(CppSQLite3Exception& e)
	{
		throw CExceptionBase(ERR_FM_CANNOTOPENDB, e.errorMessage());
	}

	return (lastShutdown>=lastStart);
}

void MarkTime(TimeMark mark)
{
	CppSQLite3DB db;
	CFeedManager::OpenDatabase(db);

	try
	{
		CTime now = CTime::GetCurrentTime();
		CString sql;
		sql.Format(_T("update configurations set %s=%d"),
			(mark==StartupTime ? _T("options1") : _T("options2")),
			(int)now.GetTime());

		db.execDML(sql);
	}
	catch(CppSQLite3Exception&)
	{
	}
}

CString DebugSql(const CString& sql)
{
	try
	{
		CppSQLite3DB db;
		CFeedManager::OpenDatabase(db);
		
		CString result = _T("\
<html><head><META http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\
<style>\
body {	font: 12px; 'Lucida Grande', Geneva, Arial, Verdana, sans-serif; } \
table { font: 12px 'Lucida Grande', Geneva, Verdana, Arial, sans-serif; border-collapse: collapse; border: black 1px solid; }\
th, td { border: black 1px solid; }\
</style>\
</head>\
<body><h3>");
		
		result += sql;
		result += _T("</h3><p><table>");

		CppSQLite3Query q = db.execQuery(sql);
		result += _T("<tr>");
        for (int fld = 0; fld < q.numFields(); fld++)
        {
			result.AppendFormat(_T("<th>%s</th>"), q.fieldName(fld));
        }
		result += _T("</tr>");

		while(!q.eof())
		{
			result += _T("<tr>");
			for (int fld = 0; fld < q.numFields(); fld++)
			{
				result.AppendFormat(_T("<td>%s</td>"), q.getStringField(fld));
			}
			result += _T("</tr>");
			q.nextRow();
		}
		result += _T("</table></body></html>");
		q.finalize();

		return result;
	}
	catch(CppSQLite3Exception& e)
	{
		throw CExceptionBase(ERR_FM_GENERICERR, e.errorMessage());
	}

	return _T("");
}

} // namespace
