#include "StdAfx.h"
#include ".\feedmanagerlibhelper.h"
#include "FeedManagerErrorCode.h"
#include "ExceptionBase.h"
#include "GMTimeLib.h"

LPCTSTR CFeedManagerLibHelper::m_newsFeedFields = 
	_T("feed_id, feed_group_id, url, name, description, image, website, last_updated,language,disabled,usage,options1,unread_count,hd_last_mod,hd_etag,update_freq,store_dur,options3,notify_newitem,limit_item,limit_item_number,keep_inactive, contentCode, parent_feed_id,news_id,use_login,login_name,login_pwd ");

void CFeedManagerLibHelper::PopulateNewsFeed(CNewsFeed* nf, CppSQLite3Query& q)
{
	nf->m_id = q.getIntField(0);
	nf->m_groupID = q.getIntField(1);
	nf->m_url = q.getStringField(2);
	nf->m_title = q.getStringField(3);
	nf->m_description = q.getStringField(4);
	nf->m_image = q.getStringField(5, _T(""));
	nf->m_website = q.getStringField(6, _T(""));
	nf->m_lastModified = q.getIntField(7, 0);
	nf->m_language = q.getStringField(8, _T(""));
	nf->m_bDisabled = (q.getIntField(9, 0)!=0);
	nf->m_usage = q.getIntField(10, 0);
	nf->m_bloglineSubId = q.getIntField(11,0);
	nf->m_unreadCount = q.getIntField(12,0);
	nf->m_httpLastModified = q.getStringField(13,_T(""));
	nf->m_httpETag = q.getStringField(14, _T(""));
	nf->m_updateFreq = q.getIntField(15,0);
	nf->m_cleanupMaxAge = q.getIntField(16,0);
	nf->m_channelStyle = q.getStringField(17, _T(""));
	nf->m_notifyNewItem = q.getIntField(18,0);
	nf->m_bLimitItems = (q.getIntField(19)!=0);
	nf->m_maxNumOfItems = q.getIntField(20, 200);
	nf->m_bKeepEvenInactive = (q.getIntField(21)!=0);
	nf->m_nContentCRC = q.getUnsignedIntField(22);
	nf->m_parent_feed_id = q.getIntField(23);
	nf->m_parent_item_id = q.getIntField(24);
	nf->m_bUseLogin = (q.getIntField(25)!=0);
	nf->m_loginName = q.getStringField(26);
	nf->m_loginPassword = q.getStringField(27);
}

LPCTSTR CFeedManagerLibHelper::m_newsItemFields = 
	_T("news_id, feed_id, url, title, description,retrieved,readtime,item_guid,mark,author,pod_url,commentRss,commentTrack ");

void CFeedManagerLibHelper::PopulateNewsItem(CNewsItem* ni, CppSQLite3Query& q)
{
	ni->m_id = q.getIntField(0);
	ni->m_feedID = q.getIntField(1);
	ni->m_url = q.getStringField(2);
	ni->m_title = q.getStringField(3);
	if(!q.fieldIsNull(4))
	{
		int nLen=0;
		const BYTE* pStream = q.getBlobField(4, nLen);
		ni->DecompressDescription(pStream, nLen);
	}
	ni->m_date = q.getIntField(5, 0);
	ni->SetReadFlag(!q.fieldIsNull(6));
	ni->m_guid = q.getStringField(7, _T(""));
	ni->m_marked = q.getIntField(8, 0);
	ni->m_author = q.getStringField(9, _T(""));
	ni->m_podCastingURL = q.getStringField(10, _T(""));
	ni->m_commentFeedURL = q.getStringField(11, _T(""));
	ni->m_bTrackComments = (q.getIntField(12)!=0);
}


LPCTSTR CFeedManagerLibHelper::m_newsWatchFields = 
	_T("watch_id,name,criteria,selective_feed,watch_flag,match_case, whole_word,all_words, front_color, bk_color,display_order, options1,unread_count, options3, options2");

void CFeedManagerLibHelper::PopulateNewsWatch(CNewsWatch* nw, CppSQLite3Query& q)
{
	nw->m_id = q.getIntField(0);
	nw->m_title = q.getStringField(1);
	nw->m_matchCriteria = q.getStringField(2);
	nw->m_bWatchSelectiveChannels = (q.getIntField(3) > 0);
	nw->m_watchFlag = q.getIntField(4);
	nw->m_matchCase = q.getIntField(5);
	nw->m_wholeWord = q.getIntField(6);
	nw->m_allWords = q.getIntField(7);
	if(!q.fieldIsNull(8))
		nw->m_txtColor = q.getIntField(8);
	if(!q.fieldIsNull(9))
		nw->m_bkColor = q.getIntField(9);
	nw->m_order = q.getIntField(10);
	nw->m_nAction = (CNewsWatch::Action)q.getIntField(11, 0);
	nw->m_unreadCount = q.getIntField(12,0);
	nw->m_watchStyle = q.getStringField(13);
	nw->m_bDisabled = (q.getIntField(14) != 0);
}

LPCTSTR CFeedManagerLibHelper::m_feedGroupFields = 
	_T("feed_group_id, name, description, options3,unread_count,options1,parent_group,options2");

void CFeedManagerLibHelper::PopulateFeedGroup(CFeedGroup* fg, CppSQLite3Query& q)
{
	fg->m_id = q.getIntField(0);
	fg->m_name = q.getStringField(1);
	fg->m_desc = q.getStringField(2);
	fg->m_website = q.getStringField(3, _T(""));
	fg->m_unreadCount = q.getIntField(4,0);
	fg->m_bDisabled = (q.getIntField(5,0)!=0);
	fg->m_parentID = q.getIntField(6,0);
	fg->m_bExpanded = (q.getIntField(7,0)!=0);

}

NewsItemPtr CFeedManagerLibHelper::GetExistingNewsItem(NewsItemPtr& item, bool bItemHasValidGuids, CppSQLite3DB& db)
{
	NewsItemPtr existingItem = NULL;
	try
	{

		//CString sql;
		//sql.Format(_T("select news_id from news_item where item_guid='%s' and feed_id = %d"),
		//		(LPCTSTR)item->m_guid, item->m_feedID);
		CppSQLite3Statement stmt;
		if(bItemHasValidGuids)
		{
			if(item->m_guid.Find(_T("http://hosted.ap.org/")) == 0)
			{
				// this is a AP's feed that generate new guids for existing items everytime @!$!#@$#$
				CString guid = item->m_guid;
				int pos = guid.Find(_T('?'));
				if(pos>0)
					guid = guid.Left(pos)+ _T("%");

				stmt = db.compileStatement(
					_T("select news_id, title, content_code, retrieved,readtime from news_item where feed_id = ? and content_code=? and title=? limit 1"));
				stmt.bind(1, item->m_feedID);
				stmt.bind(2, item->m_contentCRC);
				stmt.bind(3, (LPCTSTR)item->m_title);
			}
			else
			{
				stmt = db.compileStatement(
					_T("select news_id, title, content_code, retrieved,readtime from news_item where item_guid=? and feed_id = ? limit 1"));
				stmt.bind(1, (LPCTSTR)item->m_guid);
				stmt.bind(2, item->m_feedID);
			}
		}
		else
		{
			// if the guid is invalid, we add title to match criteria
			stmt = db.compileStatement(
				_T("select news_id, title, content_code, retrieved,readtime from news_item where item_guid=? and feed_id = ? and title=? limit 1"));
			stmt.bind(1, (LPCTSTR)item->m_guid);
			stmt.bind(2, item->m_feedID);
			stmt.bind(3,(LPCTSTR)item->m_title);
		}

		CppSQLite3Query q = stmt.execQuery();
		if(!q.eof())
		{
			CNewsItem* ni = new CNewsItem();
			ni->m_id = q.getIntField(0);
			ni->m_title = q.getStringField(1);
			ni->m_contentCRC = q.getUnsignedIntField(2, 0);
			ni->m_date = q.getIntField(3, 0);
			ni->SetReadFlag(!q.fieldIsNull(4));
			existingItem = ni;
		}
		q.finalize();
		stmt.finalize();
	}
	catch(CppSQLite3Exception& e)
	{
		throw CExceptionBase(ERR_FM_DBERROR, e.errorMessage());
	}

	return existingItem;
}


size_t CFeedManagerLibHelper::GetNewsItemIDs(std::vector<ULONG>& newsIDs, const CString& criteria, CNewsFilter* pNewsFilter, LPCTSTR orderBy)
{
	CppSQLite3DB db;
	CFeedManager::OpenDatabase(db);

	newsIDs.clear();

	try
	{
		CString sql;
		sql.Format(_T("select news_id from news_item where %s"), (LPCTSTR)criteria);
		if(pNewsFilter)
		{
			CString filterWhere = pNewsFilter->GenerateWhereClause();
			if(filterWhere.GetLength())
			{
				sql.AppendFormat(_T(" AND %s"), filterWhere);
			}
		}

		if(orderBy != NULL)
			sql.Append(orderBy);
		else if(pNewsFilter)
			sql.AppendFormat(_T(" order by %s "), (LPCTSTR)pNewsFilter->GenOrderByClause());

		CppSQLite3Query q = db.execQuery(sql);
		while(!q.eof())
		{
			newsIDs.push_back((ULONG)q.getIntField(0));
			q.nextRow();
		}
		q.finalize();
		db.close();
	}
	catch(CppSQLite3Exception& e)
	{
		throw CExceptionBase(ERR_FM_DBERROR, e.errorMessage());
	}

	return newsIDs.size();
}

size_t CFeedManagerLibHelper::RetrieveNewsFromDB(NewsItemVector& newsItems, const CString& criteria, CNewsFilter* pNewsFilter, LPCTSTR orderBy)
{
	newsItems.clear();

	CppSQLite3DB db;
	CFeedManager::OpenDatabase(db);

	try
	{
		CString sql;
		sql.Format(_T("select %s from news_item where %s"), CFeedManagerLibHelper::m_newsItemFields, (LPCTSTR)criteria);
		if(pNewsFilter)
		{
			CString filterWhere = pNewsFilter->GenerateWhereClause();
			if(filterWhere.GetLength())
			{
				sql.AppendFormat(_T(" AND %s"), filterWhere);
			}
		}
		if(orderBy != NULL)
			sql.Append(orderBy);
		else if(pNewsFilter)
			sql.AppendFormat(_T(" order by %s "), (LPCTSTR)pNewsFilter->GenOrderByClause());

		CppSQLite3Query q = db.execQuery(sql);
		while(!q.eof())
		{
			CNewsItem* ni =  new CNewsItem();
			CFeedManagerLibHelper::PopulateNewsItem(ni, q);
			newsItems.push_back(ni);
			q.nextRow();
		}
		q.finalize();
		db.close();
	}
	catch(CppSQLite3Exception& e)
	{
		throw CExceptionBase(ERR_FM_DBERROR, e.errorMessage());
	}

	return newsItems.size();
}

void CFeedManagerLibHelper::DBExec(LPCTSTR sql, TransactionSettings useTrx)
{
	CppSQLite3DB db;
	CFeedManager::OpenDatabase(db);

	try
	{
		if(useTrx==UseTransaction)
			db.execDML(_T("BEGIN IMMEDIATE TRANSACTION;"));
		db.execDML(sql);
		if(useTrx==UseTransaction)
			db.execDML(_T("commit transaction;"));
		db.close();
	}
	catch(CppSQLite3Exception& e)
	{
		throw CExceptionBase(ERR_FM_DBERROR, e.errorMessage());
	}
}

bool CFeedManagerLibHelper::GetTitle(MSXML2::IXMLDOMElementPtr spElement, CString& title)
{
	_variant_t v = spElement->getAttribute(_T("text"));
	if(v.vt == VT_BSTR)
	{
		title = v.bstrVal;
		return true;
	}

	v = spElement->getAttribute(_T("title"));
	if(v.vt == VT_BSTR)
	{
		title = v.bstrVal;
		return true;
	}

	return false;
}

bool CFeedManagerLibHelper::SafeGetAttr(MSXML2::IXMLDOMElementPtr spElement, LPCTSTR attr, CString& value)
{
	_variant_t v = spElement->getAttribute(attr);
	if(v.vt == VT_BSTR)
	{
		value = v.bstrVal;
		return true;
	}
	else
	{
		return false;
	}
}

int CFeedManagerLibHelper::MarkNewsItemUnread(long itemId, CppSQLite3DB& db)
{
	// update
	CppSQLite3Statement stmt = db.compileStatement(
		_T("update news_item set readtime = null where news_id = ? and readtime is not null"));
	stmt.bind(1, itemId);
	return stmt.execDML();
}

void CFeedManagerLibHelper::UpdateNewsFeed(CNewsFeed* nf, CppSQLite3DB& db)
{
	// update
	CppSQLite3Statement stmt = db.compileStatement(
		_T("update feed set feed_group_id=?, url=?, name=?, description=?,website=?,image=?,language=?,disabled=?, last_updated=?, options1=?, hd_last_mod=?, hd_etag=?,update_freq=?,store_dur=?,options3=?,notify_newitem=?,limit_item=?,limit_item_number=?,keep_inactive=?,contentCode=?,use_login=?,login_name=?,login_pwd=? where feed_id=?"));
	stmt.bind(1,nf->m_groupID);
	stmt.bind(2,(LPCTSTR)nf->m_url);
	stmt.bind(3,(LPCTSTR)nf->m_title);
	stmt.bind(4,(LPCTSTR)nf->m_description);
	stmt.bind(5,(LPCTSTR)nf->m_website);
	stmt.bind(6,(LPCTSTR)nf->m_image);
	stmt.bind(7,(LPCTSTR)nf->m_language);
	stmt.bind(8,nf->m_bDisabled ? 1 : 0);
	stmt.bind(9,(int)nf->m_lastModified.GetTime());
	if(nf->m_bloglineSubId)
	{
		stmt.bind(10,(int)nf->m_bloglineSubId);
	}
	else
	{
		stmt.bindNull(10);
	}
	stmt.bind(11,(LPCTSTR)nf->m_httpLastModified);
	stmt.bind(12,(LPCTSTR)nf->m_httpETag);
	stmt.bind(13,nf->m_updateFreq);
	stmt.bind(14,nf->m_cleanupMaxAge);
	stmt.bind(15,nf->m_channelStyle);
	stmt.bind(16,nf->m_notifyNewItem);
	stmt.bind(17,nf->m_bLimitItems ? 1 : 0);
	stmt.bind(18,nf->m_maxNumOfItems);
	stmt.bind(19,nf->m_bKeepEvenInactive? 1: 0);
	stmt.bind(20,nf->m_nContentCRC);
	stmt.bind(21,nf->m_bUseLogin ? 1 : 0);
	stmt.bind(22,(LPCTSTR)nf->m_loginName);
	stmt.bind(23,(LPCTSTR)nf->m_loginPassword);
	stmt.bind(24,nf->m_id);

	stmt.execDML();
}

void CFeedManagerLibHelper::SaveUnreadCount(std::vector<std::pair<long,long> >& counts, LPCTSTR tableName, LPCTSTR idName)
{
	if(counts.empty())
		return;

	CString sql;
	sql.Format(_T("update %s set unread_count=? where %s=?;"),tableName, idName);

	CppSQLite3DB db;
	CFeedManager::OpenDatabase(db);

	db.execDML(_T("BEGIN IMMEDIATE TRANSACTION;"));
	CppSQLite3Statement stmt = db.compileStatement(sql);
	for(std::vector<std::pair<long,long> >::iterator it = counts.begin(); it!=counts.end(); it++)
	{
		std::pair<long, long>& values = *it;
		stmt.bind(1, values.second);
		stmt.bind(2, values.first);
		stmt.execDML();
		stmt.reset();
	}
	stmt.finalize();
	db.execDML(_T("COMMIT TRANSACTION;"));
	db.close();

}
