#include "StdAfx.h"
#include ".\rss09xfeedparserPug.h"
#include "GMTimeLib.h"

CRss09xFeedParserPug::CRss09xFeedParserPug(std::vector<wchar_t>* pContent, pug::xml_parser* pxmlparser)
	: CNewsFeedParserImplPugXml(pContent, pxmlparser)
{
	m_newsFeed.m_format = GetFormatName(RSS09x);
}

CRss09xFeedParserPug::~CRss09xFeedParserPug(void)
{
}

//<?xml version="1.0" encoding="ISO-8859-1" ?>
//<rss version="0.91">
//	<channel>
//		<title>WriteTheWeb</title> 
//		<link>http://writetheweb.com</link> 
//		<description>News for web users that write back</description> 
//		<language>en-us</language> 
//		<copyright>Copyright 2000, WriteTheWeb team.</copyright> 
//		<managingEditor>editor@writetheweb.com</managingEditor> 
//		<webMaster>webmaster@writetheweb.com</webMaster> 
//		<image>
//			<title>WriteTheWeb</title> 
//			<url>http://writetheweb.com/images/mynetscape88.gif</url> 
//			<link>http://writetheweb.com</link> 
//			<width>88</width> 
//			<height>31</height> 
//			<description>News for web users that write back</description> 
//			</image>
//		<item>
//			<title>Giving the world a pluggable Gnutella</title> 
//			<link>http://writetheweb.com/read.php?item=24</link> 
//			<description>WorldOS is a framework on which to build programs that work like Freenet or Gnutella -allowing distributed applications using peer-to-peer routing.</description> 
//			</item>
//		<item>
//			<title>Syndication discussions hot up</title> 
//			<link>http://writetheweb.com/read.php?item=23</link> 
//			<description>After a period of dormancy, the Syndication mailing list has become active again, with contributions from leaders in traditional media and Web syndication.</description> 
//			</item>
//		<item>
//			<title>Personal web server integrates file sharing and messaging</title> 
//			<link>http://writetheweb.com/read.php?item=22</link> 
//			<description>The Magi Project is an innovative project to create a combined personal web server and messaging system that enables the sharing and synchronization of information across desktop, laptop and palmtop devices.</description> 
//			</item>
//		<item>
//			<title>Syndication and Metadata</title> 
//			<link>http://writetheweb.com/read.php?item=21</link> 
//			<description>RSS is probably the best known metadata format around. RDF is probably one of the least understood. In this essay, published on my O'Reilly Network weblog, I argue that the next generation of RSS should be based on RDF.</description> 
//			</item>
//		<item>
//			<title>UK bloggers get organised</title> 
//			<link>http://writetheweb.com/read.php?item=20</link> 
//			<description>Looks like the weblogs scene is gathering pace beyond the shores of the US. There's now a UK-specific page on weblogs.com, and a mailing list at egroups.</description> 
//			</item>
//		<item>
//			<title>Yournamehere.com more important than anything</title> 
//			<link>http://writetheweb.com/read.php?item=19</link> 
//			<description>Whatever you're publishing on the web, your site name is the most valuable asset you have, according to Carl Steadman.</description> 
//			</item>
//	</channel>
//</rss>
void CRss09xFeedParserPug::Parse()
{
	pug::xml_node channel = m_pXmlParser->document().first_element_by_path(_T("/rss/channel"));
	if(channel.empty())
		return;

	ParseChannelDC(channel);

	ParseNode(m_newsFeed.m_title, channel, _T("./title"));
	ParseNode(m_newsFeed.m_description, channel, _T("./description"));
	ParseNode(m_newsFeed.m_website, channel, _T("./link"),true);
	ParseNode(m_newsFeed.m_language, channel, _T("./language"));
	ParseNode(m_newsFeed.m_image, channel, _T("./image/url"));

	pug::xml_node buildDate = channel.first_element_by_path(_T("./lastBuildDate"));
	if(!buildDate.empty())
	{
		m_newsFeed.m_lastModified = CGMTimeHelper::ParseDate(buildDate.child_value());
	}
	else
	{
		pug::xml_node pubDate = channel.first_element_by_path(_T("./pubDate"));
		if(!pubDate.empty())
		{
			m_newsFeed.m_lastModified = CGMTimeHelper::ParseDate(pubDate.child_value());
		}
	}

	pug::xml_node ttl = channel.first_element_by_path(_T("./ttl"));
	if(!ttl.empty())
	{
		LPCTSTR value = ttl.child_value();
		if(value)
			m_newsFeed.m_ttl = _ttol(value);
	}
}

size_t CRss09xFeedParserPug::ExtractNews(NewsItemVector& newsItems)
{
	if(m_pXmlParser == NULL)
	{
		throw CExceptionBase(ERR_FM_GENERICERR, _T("NewsFeed Parser hasn't been initialized."));
	}

	newsItems.clear();

	pug::xml_node channel = m_pXmlParser->document().first_element_by_path(_T("/rss/channel"));
	if(channel.empty())
		return 0;
	pug::xml_node_list nodeList;
	channel.all_elements_by_name(_T("item"),nodeList);
	for(unsigned int i=0; i<nodeList.size(); i++)
	{
		pug::xml_node node = nodeList[i];
		NewsItemPtr newsItem(new CNewsItem());
		if(PopulateNewsItem(newsItem, node))
		{
			newsItems.push_back(newsItem);
		}
	}

	return newsItems.size();
}

bool CRss09xFeedParserPug::PopulateNewsItem(NewsItemPtr& pNews, pug::xml_node& item)
{
	if(!CNewsFeedParserImplPugXml::PopulateNewsItem(pNews, item))
		return false;

	ParseNode(pNews->m_title, item, _T("./title"));
	if(pNews->m_url.IsEmpty() || !CNewsFeedParser::m_bKeepOrigionalLink)
	{
		ParseNode(pNews->m_url, item, _T("./link"),true);
	}
	if(pNews->m_description.GetLength()==0)
		ExtractContent(pNews->m_description, item, _T("./description"));
	
	ParseNode(pNews->m_guid, item, _T("./guid"));
	if(pNews->m_guid.GetLength() == 0) // if no guid, we use link to identify the news
        pNews->m_guid = pNews->m_url;
	if(pNews->m_guid.GetLength() == 0) // if no guid and no link, we use news title as guid
		pNews->m_guid = pNews->m_title;

	pug::xml_node node = item.first_element_by_path(_T("./pubDate"));
	if(!node.empty())
	{
		pNews->m_date = CGMTimeHelper::ParseDate(node.child_value());
	}


	return true;
}
