#include "StdAfx.h"
#include ".\rss100feedparserpug.h"
#include "GMTimeLib.h"

CRss100FeedParserPug::CRss100FeedParserPug(std::vector<wchar_t>* pContent, pug::xml_parser* pxmlparser)
	: CNewsFeedParserImplPugXml(pContent, pxmlparser)
{
	m_newsFeed.m_format = GetFormatName(RSS100);
}

CRss100FeedParserPug::~CRss100FeedParserPug(void)
{
}


//<?xml version="1.0"?>
//
//<rdf:RDF 
//  xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
//  xmlns="http://purl.org/rss/1.0/"
//>
//
//  <channel rdf:about="http://www.xml.com/xml/news.rss">
//    <title>XML.com</title>
//    <link>http://xml.com/pub</link>
//    <description>
//      XML.com features a rich mix of information and services 
//      for the XML community.
//    </description>
//
//    <image rdf:resource="http://xml.com/universal/images/xml_tiny.gif" />
//
//    <items>
//      <rdf:Seq>
//        <rdf:li resource="http://xml.com/pub/2000/08/09/xslt/xslt.html" />
//        <rdf:li resource="http://xml.com/pub/2000/08/09/rdfdb/index.html" />
//      </rdf:Seq>
//    </items>
//
//  </channel>
//  
//  <image rdf:about="http://xml.com/universal/images/xml_tiny.gif">
//    <title>XML.com</title>
//    <link>http://www.xml.com</link>
//    <url>http://xml.com/universal/images/xml_tiny.gif</url>
//  </image>
//  
//  <item rdf:about="http://xml.com/pub/2000/08/09/xslt/xslt.html">
//    <title>Processing Inclusions with XSLT</title>
//    <link>http://xml.com/pub/2000/08/09/xslt/xslt.html</link>
//    <description>
//     Processing document inclusions with general XML tools can be 
//     problematic. This article proposes a way of preserving inclusion 
//     information through SAX-based processing.
//    </description>
//  </item>
//  
//  <item rdf:about="http://xml.com/pub/2000/08/09/rdfdb/index.html">
//    <title>Putting RDF to Work</title>
//    <link>http://xml.com/pub/2000/08/09/rdfdb/index.html</link>
//    <description>
//     Tool and API support for the Resource Description Framework 
//     is slowly coming of age. Edd Dumbill takes a look at RDFDB, 
//     one of the most exciting new RDF toolkits.
//    </description>
//  </item>
//
//</rdf:RDF>
void CRss100FeedParserPug::Parse()
{
	pug::xml_node root = m_pXmlParser->root_element();
	pug::xml_node channel = root.first_element_by_path(_T("./channel"));
	if(channel.empty())
		return;

	ParseChannelDC(channel);

	ParseNode(m_newsFeed.m_title, channel, _T("./title"));
	ParseNode(m_newsFeed.m_description, channel, _T("./description"));
	ParseNode(m_newsFeed.m_website, channel, _T("./link"),true);

	pug::xml_node temp = channel.first_element_by_path(_T("./image"));
	if(!temp.empty() && temp.has_attribute(_T("rdf:resource")))
	{
		m_newsFeed.m_image = temp.attribute(_T("rdf:resource")).value();
	}
}

size_t CRss100FeedParserPug::ExtractNews(NewsItemVector& newsItems)
{
	if(m_pXmlParser == NULL)
	{
		throw CExceptionBase(ERR_FM_GENERICERR, _T("NewsFeed Parser hasn't been initialized."));
	}

	newsItems.clear();

	pug::xml_node_list items;
	m_pXmlParser->root_element().all_elements_by_name(_T("item"),items);
	for(unsigned int i=0; i < items.size(); ++i)
	{
		pug::xml_node item = items[i];
		NewsItemPtr newsItem(new CNewsItem());

		CNewsFeedParserImplPugXml::PopulateNewsItem(newsItem, item);

		ParseNode(newsItem->m_title, item, _T("./title"));
		if(newsItem->m_url.IsEmpty() || !CNewsFeedParser::m_bKeepOrigionalLink)
		{
			ParseNode(newsItem->m_url, item, _T("./link"),true);
		}
		if(newsItem->m_description.GetLength()==0)
			ParseNode(newsItem->m_description, item, _T("./description"));
		
		pug::xml_node temp = item.first_element_by_path(_T("./guid"));
		if(!temp.empty())
			newsItem->m_guid = temp.child_value();
		if(newsItem->m_guid.GetLength()==0) // if no guid, use link as guid
			newsItem->m_guid = newsItem->m_url;
		if(newsItem->m_guid.GetLength()==0) // if no guid and no link, we use news title as guid
			newsItem->m_guid = newsItem->m_title;

		newsItems.push_back(newsItem);
	}

	return newsItems.size();
}
