#include "StdAfx.h"
#include "CmdLineParser.h"
#include "GNUtil.h"
#include "GMTimeLib.h"
#include "dbversion.h"
#include "FeedManagerLib.h"
#include "greatnewsconfig.h"
#include "FileUtil.h"
#include "GNResourceManager.h"
#include "resource.h"
#include "AsyncDownloadManager.h"

#include <vector>

CGreatNewsConfig g_GreatNewsConfig;

TCHAR* CGreatNewsConfig::AutoUpdateString[] = { _T("15 min"),
												_T("30 min"),
												_T("1 hour"),
												_T("2 hours"),
												_T("3 hours"),
												_T("4 hours"),
												NULL};

int CGreatNewsConfig::AutoUpdateSec[] = {	15*60,
											30*60,
											60*60,
											120*60,
											180*60,
											240*60,
											0};

TCHAR* CGreatNewsConfig::CleanupRemindString[] =  {	_T("7 days"),
									_T("14 days"),
									_T("30 days"),
									_T("60 days"),
									_T("90 days"),
									NULL};

int CGreatNewsConfig::CleanupRemindDays[] = {7,14,30,60,90,-1};

TCHAR* CGreatNewsConfig::CleanupMaxAgeString[] = {
											_T("All"),
											_T("1 week"),
											_T("2 weeks"),
											_T("1 month"),
											_T("3 months"),
											_T("6 months"),
											NULL
											};

int CGreatNewsConfig::CleanupMaxAgeDays[] = {
												0,
												7,
												14,
												30,
												90,
												180,
												-1
											};



CGreatNewsConfig::CGreatNewsConfig(void):
	m_updateFrequency(OneHour),
	m_bAutoCloseNewsList(true),
	m_bAutoSetFilterToUnread(true),
	m_bBypassTTLChecking(false),
	m_bNoChannelGroupView(false),
	m_nSelectedItemFeatures(0),
	m_bDisableTabbedBrowsing(false),
	m_bUseMozillaEngine(false),
	m_nProxyToUse(0),
	m_proxyPort(80),
	m_bNotifyDinosaurChannels(true),
	m_nDinosaurDays(60),
	m_nNewItemNotifySeconds(60),
	m_nStatsNumOfChannels(10),
	m_bStatsExcludeDisabled(false),
	m_nDoubleClickingNewsList(0),
	m_nMouseWheelSensitivity(5),
	m_bHighlightNewsWatch(true),
	m_bIgnoreDbVersion(false),
	m_bKeepOrigionalLink(false),
	m_bWorkOffline(false),
	m_maxFeedSizeInK(1000),
	m_nStartupLabel(-1),
	m_nStartupWatch(-1),
	m_bMarkChangesUnread(false),
	m_bCleanTempFilesUponExit(true),
	m_bLegacyBrowser(false),
	m_bInternetZone(true),
	m_bMultipleInstance(false),
	m_bShowTimeIn24Hr(false)
{
	m_searchTargets.Add(_T("Search Google"));
	m_searchTargetURLs.Add(_T("http://www.google.com/search?q=%s"));

	m_searchTargets.Add(_T("Google Group"));
	m_searchTargetURLs.Add(_T("http://groups.google.com/groups?q=%s"));

	m_searchTargets.Add(_T("Yahoo"));
	m_searchTargetURLs.Add(_T("http://search.yahoo.com/search?p=%s"));

	m_searchTargets.Add(_T("MSN"));
	m_searchTargetURLs.Add(_T("http://search.msn.com/results.aspx?q=%s"));

	m_searchTargets.Add(_T("Feedster"));
	m_searchTargetURLs.Add(_T("http://www.feedster.com/search.php?q=%s"));

}

CGreatNewsConfig::~CGreatNewsConfig(void)
{
}

bool CGreatNewsConfig::Init(LPCTSTR cmdLine)
{
	if(cmdLine != NULL)
	{
		CCmdLineParser parser(cmdLine);
		if(parser.HasKey(_T("db")))
		{
			m_dbFileName = parser.GetVal(_T("db"));
		}

		if(parser.HasKey(_T("feed")))
		{
			m_newFeedUrl = parser.GetVal(_T("feed"));
			m_newFeedUrl.Replace(_T("feed://http://"), _T("feed://")); // do some fix
			m_newFeedUrl.Replace(_T("http://"), _T("feed://")); // do some fix
			m_newFeedUrl.Replace(_T("feed:feed://"), _T("feed://")); // do some fix
		}

		if(parser.HasKey(_T("IgnoreDbVersion")))
		{
			m_bIgnoreDbVersion = true;
		}

		// for firefox 2.0
		CString value=cmdLine;
		if(value.Find(_T("http")) == 0)
			m_newFeedUrl = value;
	}

	m_homeDir = CalculateHomeDir();
	if(!InitializeNewInstallation(m_homeDir))
	{
		return false;
	}

	// if no db name is set, use the default db name
	if(m_dbFileName.GetLength() == 0)
	{
		m_dbFileName = m_homeDir + _T("\\")+DEFAULT_DBNAME;
		// convert to short name:
		//TCHAR lpszShortPath[MAX_PATH];
		//if(!::GetShortPathName((LPCTSTR)m_dbFileName,lpszShortPath,MAX_PATH))
		//{
		//	::MessageBox(NULL, _T("Cannot find data file"), _T("GreatNews"), MB_OK);
		//	return false;
		//}
		//m_dbFileName = 	lpszShortPath;	
	}

	CString configFile =  m_homeDir + _T("\\")+DEFAULT_CONFIG;
	m_iniFile.SetFilename(configFile);

	LoadConfig();
	LoadLicense();
	CGNUtil::m_podCastingExtensions = m_podcastingEnclosuresTypes;

	LoadSearchChannelDefinition();

	// init resource manager
	CGNResourceManager* pResMngr = CGNSingleton<CGNResourceManager>::Instance();
	pResMngr->Init(GetHomeDir()+"\\Language\\");
	pResMngr->SetLanguage(GetLanguage());

	// check db file existance
	if(!::PathFileExists(m_dbFileName))
	{
		CString msg;
		msg.Format(ResManagerPtr->GetString(IDS_DATAFILENOTFOUND), (LPCTSTR)m_dbFileName);
		::MessageBox(NULL, msg, _T("GreatNews"), MB_OK|MB_ICONSTOP);
		return false;
	}

	FeedManagerLib::Init(g_GreatNewsConfig.m_dbFileName, true);

	// check db version
	if(!m_bIgnoreDbVersion)
	{
		CString dbVer = FeedManagerLib::GetDbVersion();
		if(dbVer != DB_VERSION_STR)
		{
			CString msg;
			msg.Format(ResManagerPtr->GetString(IDS_INCOMPATFILE));
			::MessageBox(NULL, msg, _T("GreatNews"), MB_OK|MB_ICONINFORMATION);
			::ShellExecute(NULL, _T("open"), _T("GreatNewsDBMaint.exe"), NULL, g_GreatNewsConfig.GetHomeDir(), SW_SHOW);
			return false;
		}
	}

	SetupProxy();

	return true;
}

CString CGreatNewsConfig::GetConfigFileName()
{
	return m_iniFile.m_szFilename;
}

CString CGreatNewsConfig::GetHomeDir()
{
	return m_homeDir;
}

CString CGreatNewsConfig::GetPluginDir()
{
	return m_homeDir+_T("\\Plugins");
}

CString CGreatNewsConfig::GetMediaDir()
{
	return m_homeDir+_T("\\Media");
}

CString CGreatNewsConfig::GetFaviconDir()
{
	return m_homeDir+_T("\\Favicons");
}


CString CGreatNewsConfig::CalculateHomeDir()
{
	CString path;
	DWORD dwLength = GetModuleFileName( NULL, path.GetBuffer(MAX_PATH),MAX_PATH);
	path.ReleaseBuffer();
	path = path.Left(path.ReverseFind('\\'));
	if(path.Right(5).CompareNoCase(_T("Debug"))==0 || path.Right(7).CompareNoCase(_T("Release"))==0)
	{
		path = path.Left(path.ReverseFind('\\'));
	}
	return path;
}

CString CGreatNewsConfig::GetDefaultChannelOPML()
{
	return GetHomeDir()+"\\channels.opml";
}

CString CGreatNewsConfig::GetLanguage()
{
	CString language;
	m_iniFile.GetString(_T("Environment"), _T("Language"),language,_T("gn_eng.ini"));
	return language;
}
void CGreatNewsConfig::SaveLanguage(LPCTSTR languagePackName)
{
	m_iniFile.PutString(_T("Environment"), _T("Language"), languagePackName);
}

CString CGreatNewsConfig::GetWindowPlacement()
{
	CString place;
	m_iniFile.GetString(_T("Environment"), _T("WinPlacement"),place,_T(""));
	return place;
}

void CGreatNewsConfig::SetWindowPlacement(const CString& place)
{
	m_iniFile.PutString(_T("Environment"), _T("WinPlacement"), place);
}

bool CGreatNewsConfig::GetNewsListClosed()
{
	bool b = false;
	m_iniFile.GetBool(_T("Environment"), _T("NewsListClosed"), b, false);
	return b;
}

void CGreatNewsConfig::SetNewsListClosed(bool bClosed)
{
	m_iniFile.PutBool(_T("Environment"), _T("NewsListClosed"), bClosed);
}

bool CGreatNewsConfig::GetChannelTreeClosed()
{
	bool b = false;
	m_iniFile.GetBool(_T("Environment"), _T("ChannelTreeClosed"), b, false);
	return b;
}

void CGreatNewsConfig::SetChannelTreeClosed(bool bClosed)
{
	m_iniFile.PutBool(_T("Environment"), _T("ChannelTreeClosed"), bClosed);
}

int CGreatNewsConfig::GetNewsListPos()
{
	int nPos = 200;
	m_iniFile.GetInt(_T("Environment"), _T("NewsListPos"), nPos, 200);
	return nPos;
}

void CGreatNewsConfig::SetNewsListPos(int pos)
{
	m_iniFile.PutInt(_T("Environment"), _T("NewsListPos"), pos);
}

int CGreatNewsConfig::GetChannelTreePos()
{
	int nPos = 200;
	m_iniFile.GetInt(_T("Environment"), _T("ChannelTreePos"), nPos, 200);
	return nPos;
}
void CGreatNewsConfig::SetChannelTreePos(int pos)
{
	m_iniFile.PutInt(_T("Environment"), _T("ChannelTreePos"), pos);
}

int CGreatNewsConfig::GetDateFilterSelection()
{
	int n = 4;
	m_iniFile.GetInt(_T("Environment"), _T("DateFilter"), n, 4);
	return n;
}

void CGreatNewsConfig::SetDateFilterSelection(int n)
{
	m_iniFile.PutInt(_T("Environment"), _T("DateFilter"), n);
}

int CGreatNewsConfig::GetStatusFilterSelection()
{
	int n = 0;
	m_iniFile.GetInt(_T("Environment"), _T("StatusFilter"), n, 0);
	return n;
}

void CGreatNewsConfig::SetStatusFilterSelection(int n)
{
	m_iniFile.PutInt(_T("Environment"), _T("StatusFilter"), n);
}

bool CGreatNewsConfig::GetCloseToTray()
{
	bool b = false;
	m_iniFile.GetBool(_T("Options"), _T("CloseToTray"), b, false);
	return b;
}

void CGreatNewsConfig::SetCloseToTray(bool bToTray)
{
	m_iniFile.PutBool(_T("Options"), _T("CloseToTray"), bToTray);
}

bool CGreatNewsConfig::GetRemoveFromTaskBar()
{
	bool b = false;
	m_iniFile.GetBool(_T("Options"), _T("RemoveFromTaskBar"), b, false);
	return b;
}

void CGreatNewsConfig::SetRemoveFromTaskBar(bool bRemove)
{
	m_iniFile.PutBool(_T("Options"), _T("RemoveFromTaskBar"), bRemove);
}

bool CGreatNewsConfig::GetCheckNewVersionAtStartup()
{
	bool b = false;
	m_iniFile.GetBool(_T("Options"), _T("CheckNewVersion"), b, false);
	return b;
}

void CGreatNewsConfig::SetCheckNewVersionAtStartup(bool bCheck)
{
	m_iniFile.PutBool(_T("Options"), _T("CheckNewVersion"), bCheck);
}

bool CGreatNewsConfig::GetAutoUpdateAtStartup()
{
	bool b = true;
	m_iniFile.GetBool(_T("Options"), _T("AutoUpdate"), b, true);
	return b;
}

void CGreatNewsConfig::SetAutoUpdateAtStartup(bool bUpdate)
{
	m_iniFile.PutBool(_T("Options"), _T("AutoUpdate"), bUpdate);
}

int CGreatNewsConfig::GetAutoUpdateFreqSec() 
{
	return AutoUpdateSec[m_nAutoUpdateFreq];
}


int CGreatNewsConfig::GetCleanupReminderDays()
{
	return CleanupRemindDays[GetCleanupReminder()];
}

int CGreatNewsConfig::GetCleanupReminder()
{
	int n = 1;
	m_iniFile.GetInt(_T("Options"), _T("CleanupReminder"), n, 1);
	n = max(0, min(sizeof(CleanupRemindDays)/sizeof(int)-1, n));

	return n;
}

void CGreatNewsConfig::SetCleanupReminder(int n)
{
	m_iniFile.PutInt(_T("Options"), _T("CleanupReminder"), n);
}

CString CGreatNewsConfig::GetCurrentStyle()
{
	CString v;
	m_iniFile.GetString(_T("Options"), _T("Style"), v, _T("Default.css"));
	return v;
}

int CGreatNewsConfig::GetStartupScreen()
{
	return m_nStartupScreen;
}

void CGreatNewsConfig::SetStartupScreen(int n)
{
	m_nStartupScreen = n;
}

//bool CGreatNewsConfig::GetShowWelcomePage()
//{
//	bool b=true;
//	m_iniFile.GetBool(_T("Options"), _T("ShowWelcomeScreen"), b, true);
//
//	return b;
//}

//void CGreatNewsConfig::SetShowWelcomePage(bool bShow)
//{
//	m_iniFile.PutBool(_T("Options"), _T("ShowWelcomeScreen"), bShow);
//}

void CGreatNewsConfig::SetCurrentStyle(const CString& style)
{
	m_iniFile.PutString(_T("Options"), _T("Style"), style);
	m_currentStyleHTML.Empty();
}

void CGreatNewsConfig::ProcessGreatNewsTags(CString& str)
{
	// process %MEDIA% tags
	CString dir = g_GreatNewsConfig.GetHomeDir()+"/Media";
	dir.Replace(_T("\\"),_T("/"));
	str.Replace(_T("%MEDIA%"), dir);
}

LPCTSTR CGreatNewsConfig::GetCurrentStyleString()
{
	if(m_currentStyleHTML.GetLength()==0)
	{
		CString cssFileName = g_GreatNewsConfig.GetHomeDir()+"\\Media\\"+GetCurrentStyle();
		CGNUtil::ReadStringFromFile(cssFileName, m_currentStyleHTML);

		ProcessGreatNewsTags(m_currentStyleHTML);
	}

	return m_currentStyleHTML;
}

CString CGreatNewsConfig::GetStyleFileContent(const CString& styleFileName)
{
	CString content;
	CString cssFileName = g_GreatNewsConfig.GetHomeDir()+"\\Media\\"+styleFileName+".css";
	CGNUtil::ReadStringFromFile(cssFileName, content);

	ProcessGreatNewsTags(content);

	return content;
}


LPCTSTR CGreatNewsConfig::GetBuildInStyle()
{
	if(m_buildInStyle.GetLength()==0)
	{
		CString cssFileName = g_GreatNewsConfig.GetHomeDir()+"\\Media\\buildin_style.tpl";
		CGNUtil::ReadStringFromFile(cssFileName, m_buildInStyle);
		ProcessGreatNewsTags(m_buildInStyle);
	}

	return m_buildInStyle;
}

void CGreatNewsConfig::LoadConfig()
{
	m_iniFile.GetBool(_T("Usability"), _T("AutoCloseNewsList"), m_bAutoCloseNewsList, false);
	m_iniFile.GetBool(_T("Usability"), _T("AutoSetFilterToUnread"), m_bAutoSetFilterToUnread, false);
	m_iniFile.GetBool(_T("Usability"), _T("AutoMarkLastChannelAsRead"), m_bOpenNextAutoMarkLastChannelAsRead, true);
	m_iniFile.GetBool(_T("Usability"), _T("SearchAutoOpenNewsList"), m_bSearchAutoOpenNewsList, true);
	m_iniFile.GetBool(_T("Usability"), _T("ShowTooltipAfterUpdate"), m_bShowTooltipAfterUpdate, false);
	m_iniFile.GetInt(_T("Usability"), _T("BlockPopups"), m_nBlockPopups, 1);
	m_nBlockPopups = min(1, max(0,m_nBlockPopups));
	m_iniFile.GetInt(_T("Usability"), _T("MiddleClickToNewWindow"), m_nMiddleClickToNewWindow, 0);
	m_iniFile.GetInt(_T("Usability"), _T("OpenRssLinkInDefaultBrowser"), m_nOpenRssLinkInDefaultBrowser, 0);
	m_iniFile.GetBool(_T("Usability"), _T("DisableTabbedBrowsing"), m_bDisableTabbedBrowsing, false);
	m_iniFile.GetBool(_T("Usability"), _T("NoAutoGroupCollapsing"), m_bNoAutoGroupCollapse, false);
	m_iniFile.GetBool(_T("Usability"), _T("MarkChangesUnread"), m_bMarkChangesUnread, false);

	m_iniFile.GetBool(_T("Options"), _T("MarkChannelReadWhenSwitching"), m_bMarkChannelReadWhenSwitching, false);
	m_iniFile.GetBool(_T("Options"), _T("AutoUpdateChannels"), m_bAutoUpdateChannels, true);
	m_iniFile.GetInt(_T("Options"), _T("AutoUpdateFreq"), m_nAutoUpdateFreq, 1);
	m_iniFile.GetInt(_T("Options"), _T("StartupScreen"), m_nStartupScreen, 1);
	m_nStartupScreen = max(0, min(4, m_nStartupScreen));
	m_iniFile.GetInt(_T("Options"), _T("StartupLabel"), m_nStartupLabel, -1);
	m_iniFile.GetInt(_T("Options"), _T("StartupWatch"), m_nStartupWatch, -1);
	m_iniFile.GetBool(_T("Options"), _T("CleanTempFilesUponExit"), m_bCleanTempFilesUponExit, true);

	m_iniFile.GetBool(_T("Options"), _T("UpdateChannelWhenClick"), m_bUpdateWhenClick, true);
	m_iniFile.GetInt(_T("Options"), _T("PageSize"), m_nPageSize, 10);
	m_nPageSize = min(999, max(1,m_nPageSize));

	m_iniFile.GetInt(_T("Options"), _T("NextPageOpenUnread"), m_nNextPageOpenUnread, 1);
	m_nNextPageOpenUnread = min(1, max(0,m_nNextPageOpenUnread));

	m_iniFile.GetInt(_T("Options"), _T("MarkReadAutomatically"), m_nMarkReadAutomatically, 0);
	m_nMarkReadAutomatically = min(1, max(0,m_nMarkReadAutomatically));

	m_iniFile.GetInt(_T("Options"), _T("SecondsToMarkItemRead"), m_nSecondsToMarkItemRead, 2);
	m_nSecondsToMarkItemRead = min(999, max(0,m_nSecondsToMarkItemRead));

	m_iniFile.GetInt(_T("Options"), _T("SelectedItemFeatures"), m_nSelectedItemFeatures, 
			CNewsItem::LabelThis|CNewsItem::Arthur|CNewsItem::Delicious);
	m_iniFile.GetBool(_T("Options"), _T("HighlightNewsWatch"), m_bHighlightNewsWatch, true);
	m_iniFile.GetString(_T("Options"), _T("PodcastingEnclosuresTypes"), m_podcastingEnclosuresTypes, 
			_T(".mp3;.mp4;.aac;.ogg.wmv;.wma;.avi;.wav;.mpeg;.mov;.pdf;"));
	m_iniFile.GetBool(_T("Options"), _T("NotifyDinosaurChannels"), m_bNotifyDinosaurChannels, true);
	m_iniFile.GetInt(_T("Options"), _T("DinosaurDays"), m_nDinosaurDays, 60);
	m_iniFile.GetInt(_T("Options"), _T("NewItemPopupSeconds"), m_nNewItemNotifySeconds, 60);
	m_iniFile.GetInt(_T("Options"), _T("DoubleClickingNewsList"), m_nDoubleClickingNewsList, 0);
	m_iniFile.GetBool(_T("Options"), _T("UseOrigionalLink"), m_bKeepOrigionalLink, false);
	CNewsFeedParser::m_bKeepOrigionalLink = m_bKeepOrigionalLink;
	m_iniFile.GetBool(_T("Options"), _T("WorkOffline"), m_bWorkOffline, false);
	m_iniFile.GetInt(_T("Options"), _T("ReadingPanePosition"), m_nReadingPanePos, 0);

	// connection related options
	m_iniFile.GetInt(_T("Options"), _T("ProxyToUse"), m_nProxyToUse,0);
	m_iniFile.GetString(_T("Options"), _T("ProxyServerUrl"), m_proxyURL);
	m_iniFile.GetInt(_T("Options"), _T("ProxyPort"), m_proxyPort, 80);
	m_iniFile.GetString(_T("Options"), _T("ProxyServerUser"), m_proxyUserName);
	m_iniFile.GetString(_T("Options"), _T("ProxyServerPassword"), m_proxyPassword);
	m_iniFile.GetString(_T("Options"), _T("ProxyServerBypass"), m_proxyBypass, _T("<local>"));

	// statistics related options
	m_iniFile.GetInt(_T("Statistics"), _T("NumOfChannels"), m_nStatsNumOfChannels,10);
	m_iniFile.GetBool(_T("Statistics"), _T("ExcludeDisabledChannels"), m_bStatsExcludeDisabled, false);

	// advanced options
	m_iniFile.GetBool(_T("Advanced"), _T("BypassTTL"), m_bBypassTTLChecking, false);
	m_iniFile.GetBool(_T("Advanced"), _T("NoChannelGroupView"), m_bNoChannelGroupView, false);
	m_iniFile.GetBool(_T("Advanced"), _T("UseMozillaEngine"), m_bUseMozillaEngine, false);
	m_iniFile.GetInt(_T("Advanced"), _T("MouseWheelSensitivity"), m_nMouseWheelSensitivity, 5);
	m_iniFile.GetBool(_T("Advanced"), _T("NoFavicons"), m_bNoFavicons, false);
	m_iniFile.GetString(_T("Advanced"), _T("ExternalBrowser"), CGNUtil::m_browserPath, _T(""));
	m_iniFile.GetInt(_T("Advanced"), _T("MaxFeedSizeInK"), m_maxFeedSizeInK, 1000);
	m_iniFile.GetString(_T("Advanced"), _T("TempDirectory"), m_tempDirectory, _T(""));
	m_iniFile.GetBool(_T("Advanced"), _T("LegacyBrowser"), m_bLegacyBrowser, false);
	m_iniFile.GetBool(_T("Advanced"), _T("InternetZone"), m_bInternetZone, true);
	m_iniFile.GetBool(_T("Advanced"), _T("MultipleInstance"), m_bMultipleInstance, false);
	m_iniFile.GetBool(_T("Advanced"), _T("ShowTimeIn24Hr"), m_bShowTimeIn24Hr, false);
	CGMTimeHelper::m_bShowTimeIn24Hr = m_bShowTimeIn24Hr;
	
}


void CGreatNewsConfig::SaveConfig()
{
	m_iniFile.PutBool(_T("Usability"), _T("AutoCloseNewsList"), m_bAutoCloseNewsList);
	m_iniFile.PutBool(_T("Usability"), _T("AutoSetFilterToUnread"), m_bAutoSetFilterToUnread);
	m_iniFile.PutBool(_T("Usability"), _T("AutoMarkLastChannelAsRead"), m_bOpenNextAutoMarkLastChannelAsRead);
	m_iniFile.PutBool(_T("Usability"), _T("SearchAutoOpenNewsList"), m_bSearchAutoOpenNewsList);
	m_iniFile.PutBool(_T("Usability"), _T("ShowTooltipAfterUpdate"), m_bShowTooltipAfterUpdate);
	m_iniFile.PutInt(_T("Usability"), _T("MiddleClickToNewWindow"), m_nMiddleClickToNewWindow);
	m_iniFile.PutInt(_T("Usability"), _T("BlockPopups"), m_nBlockPopups);
	m_iniFile.PutInt(_T("Usability"), _T("OpenRssLinkInDefaultBrowser"), m_nOpenRssLinkInDefaultBrowser);
	m_iniFile.PutBool(_T("Usability"), _T("DisableTabbedBrowsing"), m_bDisableTabbedBrowsing);
	m_iniFile.PutBool(_T("Usability"), _T("NoAutoGroupCollapsing"), m_bNoAutoGroupCollapse);
	m_iniFile.PutBool(_T("Usability"), _T("MarkChangesUnread"), m_bMarkChangesUnread);

	m_iniFile.PutBool(_T("Options"), _T("MarkChannelReadWhenSwitching"), m_bMarkChannelReadWhenSwitching);
	m_iniFile.PutBool(_T("Options"), _T("UpdateChannelWhenClick"), m_bUpdateWhenClick);
	m_iniFile.PutInt(_T("Options"), _T("SecondsToMarkItemRead"), m_nSecondsToMarkItemRead);
	m_iniFile.PutInt(_T("Options"), _T("PageSize"), m_nPageSize);
	m_iniFile.PutInt(_T("Options"), _T("NextPageOpenUnread"), m_nNextPageOpenUnread);
	m_iniFile.PutInt(_T("Options"), _T("MarkReadAutomatically"), m_nMarkReadAutomatically);
	m_iniFile.PutInt(_T("Options"), _T("SelectedItemFeatures"), m_nSelectedItemFeatures);
	m_iniFile.PutBool(_T("Options"), _T("HighlightNewsWatch"), m_bHighlightNewsWatch);
	m_iniFile.PutString(_T("Options"), _T("PodcastingEnclosuresTypes"), m_podcastingEnclosuresTypes); 

	m_iniFile.PutBool(_T("Options"), _T("NotifyDinosaurChannels"), m_bNotifyDinosaurChannels);
	m_iniFile.PutInt(_T("Options"), _T("DinosaurDays"), m_nDinosaurDays);
	m_iniFile.PutInt(_T("Options"), _T("NewItemPopupSeconds"), m_nNewItemNotifySeconds);
	m_iniFile.PutInt(_T("Options"), _T("DoubleClickingNewsList"), m_nDoubleClickingNewsList);
	m_iniFile.PutBool(_T("Options"), _T("UseOrigionalLink"), m_bKeepOrigionalLink);
	m_iniFile.PutBool(_T("Options"), _T("WorkOffline"), m_bWorkOffline);
	m_iniFile.PutInt(_T("Options"), _T("ReadingPanePosition"), m_nReadingPanePos);

	m_iniFile.PutBool(_T("Options"), _T("AutoUpdateChannels"), m_bAutoUpdateChannels);
	m_iniFile.PutInt(_T("Options"), _T("AutoUpdateFreq"), m_nAutoUpdateFreq);
	m_iniFile.PutInt(_T("Options"), _T("StartupScreen"), m_nStartupScreen);
	m_iniFile.PutInt(_T("Options"), _T("StartupLabel"), m_nStartupLabel);
	m_iniFile.PutInt(_T("Options"), _T("StartupWatch"), m_nStartupWatch);
	m_iniFile.PutBool(_T("Options"), _T("CleanTempFilesUponExit"), m_bCleanTempFilesUponExit);

	// connection related options
	m_iniFile.PutInt(_T("Options"), _T("ProxyToUse"), m_nProxyToUse);
	m_iniFile.PutString(_T("Options"), _T("ProxyServerUrl"), m_proxyURL);
	m_iniFile.PutInt(_T("Options"), _T("ProxyPort"), m_proxyPort);
	m_iniFile.PutString(_T("Options"), _T("ProxyServerUser"), m_proxyUserName);
	m_iniFile.PutString(_T("Options"), _T("ProxyServerPassword"), m_proxyPassword);
	m_iniFile.PutString(_T("Options"), _T("ProxyServerBypass"), m_proxyBypass);

	// statistics related options
	m_iniFile.PutInt(_T("Statistics"), _T("NumOfChannels"), m_nStatsNumOfChannels);
	m_iniFile.PutBool(_T("Statistics"), _T("ExcludeDisabledChannels"), m_bStatsExcludeDisabled);
}


//
// advanced options
//
int CGreatNewsConfig::GetChannelUpdateTimeOut()
{
	int n = 180;
	m_iniFile.GetInt(_T("Advanced"), _T("ChannelUpdateTimeout"), n, 180);
	n = min(3600, max(10,n));
	return n;
}

int CGreatNewsConfig::GetNumOfConcurrentUpdate()
{
	int n = 10;
	m_iniFile.GetInt(_T("Advanced"), _T("NumOfConcurrentUpdate"), n, 10);
	n = min(30, max(1,n));

	return n;
}


CTime CGreatNewsConfig::GetCleanupLastRun()
{
	CString time;
	m_iniFile.GetString(_T("Environment"),_T("CleanupLastRun"), time, _T(""));
	if(time.GetLength() == 0)
	{
		SetCleanupLastRunNow();
		return CGMTimeHelper::GetCurrentSysTime();
	}
	else
	{
		return CGMTimeHelper::StringToCTime(time);
	}

}

void CGreatNewsConfig::SetCleanupLastRunNow()
{
	CString time;
	time = CGMTimeHelper::FormatSqlDate(CGMTimeHelper::GetCurrentSysTime());
	m_iniFile.PutString(_T("Environment"),_T("CleanupLastRun"), time);
}


int CGreatNewsConfig::GetColWidthTitle()
{
	int n = 0;
	m_iniFile.GetInt(_T("Environment"), _T("ColWidthTitle"), n, 400);
	return min(800,max(n,1));
}

void CGreatNewsConfig::SetColWidthTitle(int n)
{
	m_iniFile.PutInt(_T("Environment"), _T("ColWidthTitle"), n);
}

int CGreatNewsConfig::GetColWidthDate()
{
	int n = 0;
	m_iniFile.GetInt(_T("Environment"), _T("ColWidthDate"), n, 130);
	return min(200,max(n,1));
}

void CGreatNewsConfig::SetColWidthDate(int n)
{
	m_iniFile.PutInt(_T("Environment"), _T("ColWidthDate"), n);
}

int CGreatNewsConfig::GetColWidthAuthor()
{
	int n = 0;
	m_iniFile.GetInt(_T("Environment"), _T("ColWidthAuthor"), n, 150);
	return min(400,max(n,1));
}

void CGreatNewsConfig::SetColWidthAuthor(int n)
{
	m_iniFile.PutInt(_T("Environment"), _T("ColWidthAuthor"), n);
}

int CGreatNewsConfig::GetColWidthChannel()
{
	int n = 0;
	m_iniFile.GetInt(_T("Environment"), _T("ColWidthChannel"), n, 120);
	return min(400,max(n,1));
}

void CGreatNewsConfig::SetColWidthChannel(int n)
{
	m_iniFile.PutInt(_T("Environment"), _T("ColWidthChannel"), n);
}

int CGreatNewsConfig::GetCleanupKeepItems()
{
	int n = 1;
	m_iniFile.GetInt(_T("Options"), _T("CleanupKeepItems"), n, 1);
	return n;
}

void CGreatNewsConfig::SetCleanupKeepItems(bool bKeep)
{
	m_iniFile.PutInt(_T("Options"), _T("CleanupKeepItems"), bKeep);
}

int CGreatNewsConfig::GetCleanupNumOfItemsToKeep()
{
	int n = 200;
	m_iniFile.GetInt(_T("Options"), _T("CleanupNumOfItemsToKeep"), n, 200);
	return n;
}

void CGreatNewsConfig::SetCleanupNumOfItemsToKeep(int num)
{
	m_iniFile.PutInt(_T("Options"), _T("CleanupNumOfItemsToKeep"), num);
}

int CGreatNewsConfig::GetCleanupDefault()
{
	int n = 1;
	m_iniFile.GetInt(_T("Options"), _T("CleanupDefault"), n, 1);
	return n;
}
void CGreatNewsConfig::SetCleanupDefault(int n)
{
	m_iniFile.PutInt(_T("Options"), _T("CleanupDefault"), n);
}

bool CGreatNewsConfig::GetCleanupDeleteUnread()
{
	bool b = false;
	m_iniFile.GetBool(_T("Options"), _T("CleanupDeleteUnread"), b, false);
	return b;
}

void CGreatNewsConfig::SetCleanupDeleteUnread(bool b)
{
	m_iniFile.PutBool(_T("Options"), _T("CleanupDeleteUnread"), b);
}

bool CGreatNewsConfig::GetCleanupDeleteTemp()
{
	bool b = false;
	m_iniFile.GetBool(_T("Options"), _T("CleanupDeleteTempFiles"), b, false);
	return b;
}

void CGreatNewsConfig::SetCleanupDeleteTemp(bool b)
{
	m_iniFile.PutBool(_T("Options"), _T("CleanupDeleteTempFiles"), b);
}

bool CGreatNewsConfig::GetCleanupDeleteLabeled()
{
	bool b = false;
	m_iniFile.GetBool(_T("Options"), _T("CleanupDeleteLabeled"), b, false);
	return b;
}
void CGreatNewsConfig::SetCleanupDeleteLabeled(bool bLabeled)
{
	m_iniFile.PutBool(_T("Options"), _T("CleanupDeleteLabeled"), bLabeled);
}

bool CGreatNewsConfig::GetCleanupCompress()
{
	bool b = false;
	m_iniFile.GetBool(_T("Options"), _T("CleanupCompress"), b, false);
	return b;
}

void CGreatNewsConfig::SetCleanupCompress(bool bCompress)
{
	m_iniFile.PutBool(_T("Options"), _T("CleanupCompress"), bCompress);
}



void CGreatNewsConfig::RegisterFeedProtocalHandler()
{
	CComPtr<IRegistrar> spRegistrar;
	spRegistrar.CoCreateInstance(CLSID_Registrar);
	ATLASSERT(spRegistrar != NULL);
	TCHAR szModule[_MAX_PATH];
	GetModuleFileName(NULL, szModule, _MAX_PATH);
	spRegistrar->AddReplacement(_T("MODULE"), szModule);

	HRESULT hr = spRegistrar->ResourceRegister(szModule,IDR_MAINREGISTRY,OLESTR("REGISTRY"));

	return;
}

void CGreatNewsConfig::LoadSearchChannelDefinition()
{
	std::vector<CString> keys;
	m_iniFile.EnumSections(keys, _T("SearchChannelDefinition"));

	// if no search channel definition was found, we populate it with default ones
	if(keys.empty())
	{
		m_iniFile.PutString(_T("SearchChannelDefinition"), 
							_T("Yahoo! - Company News via RSS"),
							_T("http://finance.yahoo.com/rss/headline?s=%KEYWORD%"));
		m_iniFile.PutString(_T("SearchChannelDefinition"), 
							_T("Feedster"),
							_T("http://www.feedster.com/search.php?q=%KEYWORD%&sort=date&content=full&type=rss&limit=20"));
		m_iniFile.PutString(_T("SearchChannelDefinition"), 
							_T("MSN"), 
							_T("http://search.msn.com/results.aspx?q=%KEYWORD%&format=rss"));
		m_iniFile.PutString(_T("SearchChannelDefinition"), 
							_T("Flickr"),
							_T("http://www.flickr.com/services/feeds/photos_public.gne?tags=%KEYWORD%&format=rss_200"));
		m_iniFile.PutString(_T("SearchChannelDefinition"), 
							_T("Google Blog Search"),
							_T("http://blogsearch.google.com/blogsearch_feeds?q=%KEYWORD%&btnG=Search+Blogs&num=50&output=rss"));
		m_iniFile.PutString(_T("SearchChannelDefinition"), 
							_T("Google Blog Search By Date"),
							_T("http://blogsearch.google.com/blogsearch_feeds?q=%KEYWORD%&btnG=Search+Blogs&num=50&output=rss&scoring=d"));
		m_iniFile.Flush();

		m_iniFile.EnumSections(keys, _T("SearchChannelDefinition"));
	}

	SearchChannelDefVector& defs = CSearchChannelDef::m_searchChannelDefs;
	for(std::vector<CString>::iterator it=keys.begin(); it!=keys.end();it++)
	{
		CString value;
		m_iniFile.GetString(_T("SearchChannelDefinition"), *it, value);
		defs.push_back(	CSearchChannelDef(*it, value) );
	}
}

bool CGreatNewsConfig::InitializeNewInstallation(const CString& homeDir)
{
	try
	{
		if(!CFileUtil::FileExist(homeDir+_T("\\newsfeed.db")))
		{
			if(!CFileUtil::Rename(homeDir+_T("\\newsfeed_upgrade.db"), homeDir+_T("\\newsfeed.db")))
			{
				::MessageBox(NULL, _T("GreatNews failed to initialize. No valid newsfeed.db was detected."), 
					_T("GreatNews"), MB_OK|MB_ICONSTOP);
				return false;
			}
		}

		if(!CFileUtil::FileExist(homeDir+_T("\\greatnews.ini")))
		{
			if(!CFileUtil::Rename(homeDir+_T("\\greatnews_upgrade.ini"), homeDir+_T("\\greatnews.ini")))
			{
				::MessageBox(NULL, _T("GreatNews failed to initialize. No valid greatnews.ini was detected."), 
					_T("GreatNews"), MB_OK|MB_ICONSTOP);
				return false;
			}
		}

		// now cleanup the backups
		if(CFileUtil::FileExist(homeDir+_T("\\newsfeed_upgrade.db")))
			CFileUtil::DeleteFile(homeDir+_T("\\newsfeed_upgrade.db"));
		if(CFileUtil::FileExist(homeDir+_T("\\greatnews_upgrade.ini")))
			CFileUtil::DeleteFile(homeDir+_T("\\greatnews_upgrade.ini"));
	}
	catch(...)
	{
		return false;
	}

	return true;
}

bool CGreatNewsConfig::SetupProxy()
{
	INTERNET_PROXY_INFO ipi = {0};
	DWORD dwSize2 = sizeof(ipi);

	// If running under Wine, suppress error messages - Its Wininet.dll implementation is incomplete.
	bool bWine = false;
	static const char * (CDECL *pwine_get_version)(void);
	HMODULE hntdll = GetModuleHandle(_T("ntdll.dll"));
	if (hntdll)
	{
		pwine_get_version = (const char *(__cdecl *)(void))GetProcAddress(hntdll, "wine_get_version");
		if (pwine_get_version)
			bWine = true;
	}
	
	if(m_nProxyToUse == 0)	// use default proxy settings
	{
		ipi.dwAccessType = INTERNET_OPEN_TYPE_PRECONFIG ;
		if(!InternetSetOption(NULL, INTERNET_OPTION_PROXY, &ipi, dwSize2))
			if (!bWine)
				MessageBox(NULL, CGNUtil::GetLastErrorMsg(_T("Failed to change proxy settings to default.")), _T("GreatNews"), MB_OK|MB_ICONINFORMATION);
	}
	else
	{
		m_proxyURL.Trim();
		if(m_proxyURL.GetLength())
		{
			m_combinedProxyURL.Format(_T("%s:%d"), m_proxyURL, m_proxyPort);
			ipi.dwAccessType = INTERNET_OPEN_TYPE_PROXY ;
			ipi.lpszProxy = (LPCTSTR)m_combinedProxyURL;
			if(m_proxyBypass.GetLength())
				ipi.lpszProxyBypass = (LPCTSTR)m_proxyBypass;
			if(!InternetSetOption(NULL, INTERNET_OPTION_PROXY, &ipi, dwSize2))
				if (!bWine)
					MessageBox(NULL, CGNUtil::GetLastErrorMsg(_T("Failed to change proxy settings.")), _T("GreatNews"), MB_OK|MB_ICONINFORMATION);
		}

		if(m_proxyUserName.GetLength())
		{
			if(!InternetSetOption(NULL, INTERNET_OPTION_PROXY_USERNAME , (LPVOID)(LPCTSTR)m_proxyUserName, m_proxyUserName.GetLength()))
				if (!bWine)
					MessageBox(NULL, CGNUtil::GetLastErrorMsg(_T("Failed to change proxy user name.")), _T("GreatNews"), MB_OK|MB_ICONINFORMATION);

			if(m_proxyPassword.GetLength())
			{
				if(!InternetSetOption(NULL, INTERNET_OPTION_PROXY_PASSWORD , (LPVOID)(LPCTSTR)m_proxyPassword, m_proxyPassword.GetLength()))
					if (!bWine)
						MessageBox(NULL, CGNUtil::GetLastErrorMsg(_T("Failed to change proxy password.")), _T("GreatNews"), MB_OK|MB_ICONINFORMATION);
			}
		}


	}

	CGNSingleton<CAsyncDownloadManager>::Instance()->Init(m_nProxyToUse == 1,
															m_combinedProxyURL,
															m_proxyBypass,
															m_proxyUserName,
															m_proxyPassword);

	return true;
}



int CGreatNewsConfig::GetNewsListSortColumn()
{
	int n = -1;
	m_iniFile.GetInt(_T("Environment"), _T("NewsListSortColumn"), n, -1);
	return n;
}

bool CGreatNewsConfig::GetNewsListSortAscending()
{
	bool b = false;
	m_iniFile.GetBool(_T("Environment"), _T("NewsListSortAscending"), b, false);
	return b;
}

void CGreatNewsConfig::SetNewsListSortColumn(int col)
{
	m_iniFile.PutInt(_T("Environment"), _T("NewsListSortColumn"), col);
}

void CGreatNewsConfig::SetNewsListSortAscending(bool bAscending)
{
	m_iniFile.PutBool(_T("Environment"), _T("NewsListSortAscending"), bAscending);
}



bool CGreatNewsConfig::LoadLicense()
{
	//
	// load license file
	//
	CString licFileName = GetHomeDir()+"\\license.key";
	std::vector<BYTE> licData;
	if(!CGNUtil::ReadBinFile(licFileName, licData))
		return false;

	//
	// load public key
	//
	std::vector<BYTE> pk;
    LPBYTE pData = NULL;
    HGLOBAL hg = NULL;
	HRSRC hRes = ::FindResource(NULL, MAKEINTRESOURCE(IDR_PUBLICKEY), _T("PK"));
	if ((hRes) && (hg = ::LoadResource(NULL, hRes)))
    {
		int nSize = ::SizeofResource(NULL, hRes);
		if (nSize && (pData = (LPBYTE)::LockResource(hg)))
        {
			pk.assign(pData, pData+nSize);
        }
        FreeResource(hg);
    } 
	if(pk.empty())
		return false; // failed to load public key

	//
	// decrypt license
	//

	return true;
}



