#include "StdAfx.h"
#include ".\updatenewsthread.h"

CUpdateNewsThread::CUpdateNewsThread(HWND notifyWnd, int type, int id) :
	m_hwndToNotify(notifyWnd), m_updateType(type), m_updateID(id)
{
}

CUpdateNewsThread::~CUpdateNewsThread(void)
{
}

DWORD CUpdateNewsThread::Run()
{
	// CWaitCursor wc(1,IDC_APPSTARTING);

	HRESULT hRes = ::CoInitialize(NULL);

	try
	{
		if(m_updateType == FEED)
		{
			NewsFeedPtr feed = new CNewsFeed();
			feed->Init(m_updateID);
			UpdateChannel(feed);
		}
		else if(m_updateType == GROUP)
		{
			FeedGroupPtr group = new CFeedGroup();
			group->Init(m_updateID);
			int finished = 0;
			UpdateGroup(group, finished);
		}
		else if(m_updateType == ALL)
		{
			int total = CNewsFeed::GetNewsFeedCount();
			int finished = 0;
			FeedGroupPtr spRootFeedGroup = CFeedGroup::GetRootFeedGroup();
			FeedGroupVector childGroups = spRootFeedGroup->GetChildGroups();
			for(FeedGroupVector::iterator it = childGroups.begin(); it != childGroups.end(); it++)
			{
				UpdateGroup(*it, finished, total);
			}
		}
	}
	CATCH_ALL_ERROR()

	::CoUninitialize();

	::PostMessage(m_hwndToNotify, MM_UPDATEDONE, 0 ,0);

	return 0;
}

void CUpdateNewsThread::UpdateGroup(FeedGroupPtr group,int& finished, int total)
{
	NewsFeedVector feeds = group->GetNewsFeeds(false);
	if(total < 0)
		total = feeds.size();

	for(NewsFeedVector::iterator it = feeds.begin(); it!=feeds.end(); it++)
	{
		finished++;
		UpdateChannel(*it, finished, total);
	}
}

void CUpdateNewsThread::UpdateChannel(NewsFeedPtr feed, int finished, int total)
{
	try
	{
		CString text;
		text.Format(_T("Updating channel %s..."), feed->m_title);
		LPTSTR p = new TCHAR[text.GetLength()+1];
		_tcscpy(p, (LPCTSTR)text);
		::PostMessage(m_hwndToNotify, MM_UPDATEPROGRESS, (int)((float)finished/total*100) ,(LPARAM)p);

		int numOfNewItems = feed->RetrieveNewsToDB();
		if(numOfNewItems > 0 && IsWindow(m_hwndToNotify))
		{
			::PostMessage(m_hwndToNotify, MM_UPDATEDATAREADY, feed->m_id ,0);
		}
	}
	catch(const CExceptionBase& e)
	{
		// don't do anything if error happened in this channel
		AtlTrace(e.GetErrorMsg());
	}
}
