#if !defined(_SIGNALH_)
#define _SIGNALH_

#include <vector>
#include <corexcept.h>
#include <criticsec.h>
#include <lock.h>

namespace ix
{
	typedef void*	conn_t;

	struct SingleThreaded
	{	
		typedef FakeLock LockType;
	};

	struct MultiThreaded
	{	
		typedef CriticalSection LockType;
	};

	//Signal<x> class forward declarations:
	template <typename ThreadModel = SingleThreaded> class Signal0;
	template <typename P1, typename ThreadModel = SingleThreaded> class Signal1;
	template <typename P1, typename P2, typename ThreadModel = SingleThreaded> class Signal2;
	template <typename P1, typename P2, typename P3, typename ThreadModel = SingleThreaded> class Signal3;
	template <typename P1, typename P2, typename P3, typename P4, typename ThreadModel = SingleThreaded> class Signal4;
	template <typename P1, typename P2, typename P3, typename P4, typename P5, typename ThreadModel = SingleThreaded> class Signal5;

	
	//Connection<x>Base classes	
	template <typename R>
	class Connection0Base
	{
	public:				
		virtual R emit() = 0;				
		virtual Connection0Base* clone() = 0;
	};

	template <typename R, typename P1>
	class Connection1Base
	{
	public:		
		virtual R emit(P1 param) = 0;	
		virtual Connection1Base* clone() = 0;
	};

	template <typename R, typename P1, typename P2>
	class Connection2Base
	{
	public:
		virtual R emit(P1 p1, P2 p2) = 0;			
		virtual Connection2Base* clone() = 0;
	};

	template <typename R, typename P1, typename P2, typename P3>
	class Connection3Base
	{
	public:
		virtual R emit(P1 p1, P2 p2, P3 p3) = 0;			
		virtual Connection3Base* clone() = 0;
	};

	template <typename R, typename P1, typename P2, typename P3, typename P4>
	class Connection4Base
	{
	public:
		virtual R emit(P1, P2, P3, P4) = 0;		
		virtual Connection4Base* clone() = 0;
	};

	template <typename R, typename P1, typename P2, typename P3, typename P4, typename P5>
	class Connection5Base
	{
	public:
		virtual R emit(P1, P2, P3, P4, P5) = 0;			
		virtual Connection5Base* clone() = 0;
	};

	//genreic Connection<x> classes:

	template <typename FUNC, typename R>
	class Connection0 : public Connection0Base<R>
	{				
	public:
		Connection0(FUNC func):_func(func){}
		Connection0Base<R>* clone(){return new Connection0(_func);}
		R emit(){return _func();}			
	private:
		FUNC _func;	
	};

	template <typename FUNC, typename R, typename P1>
	class Connection1 : public Connection1Base<R, P1>
	{				
	public:
		Connection1(FUNC func):_func(func){}
		Connection1Base<R, P1>* clone(){return new Connection1(_func);}
		R emit(P1 p1){return _func(p1);}			
	private:
		FUNC _func;	
	};

	template <typename FUNC, typename R, typename P1, typename P2>
	class Connection2 : public Connection2Base<R, P1, P2>
	{				
	public:
		Connection2(FUNC func): _func(func)	{}
		Connection2Base<R, P1, P2>* clone(){return new Connection2(_func);}
		R emit(P1 p1, P2 p2)	{return _func(p1, p2);	}			
	private:
		FUNC _func;	
	};

	template <typename FUNC, typename R, typename P1, typename P2, typename P3>
	class Connection3 : public Connection3Base<R, P1, P2, P3>
	{				
	public:
		Connection3(FUNC func): _func(func){}
		Connection3Base<R, P1, P2, P3>* clone(){return new Connection3(_func);}
		R emit(P1 p1, P2	p2, P3 p3){return _func(p1, p2, p3);}			
	private:
		FUNC _func;	
	};

	template <typename FUNC, typename R, typename P1, typename P2, typename P3, typename P4>
	class Connection4 : public Connection4Base<R, P1, P2, P3, P4>
	{				
	public:
		Connection4(FUNC func): _func(func){}
		Connection4Base<R, P1, P2, P3, P4>* clone(){return new Connection4(_func);}
		R emit(P1 p1, P2	p2, P3 p3, P4 p4){return _func(p1, p2, p3, p4);}			
	private:
		FUNC _func;	
	};

	template <typename FUNC, typename R, typename P1, typename P2, typename P3, typename P4, typename P5>
	class Connection5 : public Connection5Base<R, P1, P2, P3, P4, P5>
	{				
	public:
		Connection5(FUNC func): _func(func)	{}
		Connection5Base<R, P1, P2, P3, P4, P5>* clone(){return new Connection5(_func);}
		R emit(P1 p1, P2	p2, P3 p3, P4 p4, P5 p5){return _func(p1, p2, p3, p4, p5);}			
	private:
		FUNC _func;	
	};

	

	//member function connection classes:
	template <typename T, typename R>
	class MemberConnection0 : public Connection0Base<R>
	{		
		typedef R (T::*memberFunc)();
	public:
		MemberConnection0(T* object, memberFunc member):_object(object), _memberFunc(member){}
		Connection0Base<R>* clone(){return new MemberConnection0(_object, _memberFunc);}
		R emit(){(_object->*_memberFunc)();}			
	private:		
		T* _object;
		memberFunc _memberFunc;	
	};

	template <typename T, typename R, typename P1>
	class MemberConnection1 : public Connection1Base<R, P1>
	{		
		typedef R (T::*memberFunc)(P1);
	public:
		MemberConnection1(T* object, memberFunc member):_object(object), _memberFunc(member){}
		Connection1Base<R, P1>* clone(){return new MemberConnection1(_object, _memberFunc);}
		R emit(P1 p1){return (_object->*_memberFunc)(p1);}			
	private:		
		T* _object;
		memberFunc _memberFunc;	
	};

	template <typename T, typename R, typename P1, typename P2>
	class MemberConnection2 : public Connection2Base<R, P1, P2>
	{		
		typedef R (T::*memberFunc)(P1, P2);
	public:
		MemberConnection2(T* object, memberFunc member):_object(object), _memberFunc(member){}
		Connection2Base<R, P1, P2>* clone(){return new MemberConnection2(_object, _memberFunc);}
		R emit(P1 p1, P2 p2){return (_object->*_memberFunc)(p1, p2);}			
	private:		
		T* _object;
		memberFunc _memberFunc;	
	};

	template <typename T, typename R, typename P1, typename P2, typename P3>
	class MemberConnection3 : public Connection3Base<R, P1, P2, P3>
	{		
		typedef R (T::*memberFunc)(P1, P2, P3);
	public:
		MemberConnection3(T* object, memberFunc member):_object(object), _memberFunc(member){}
		Connection3Base<R, P1, P2, P3>* clone(){return new MemberConnection3(_object, _memberFunc);}
		R emit(P1 p1, P2 p2, P3 p3){return (_object->*_memberFunc)(p1, p2, p3);}			
	private:		
		T* _object;
		memberFunc _memberFunc;	
	};

	template <typename T, typename R, typename P1, typename P2, typename P3, typename P4>
	class MemberConnection4 : public Connection4Base<R, P1, P2, P3, P4>
	{		
		typedef R (T::*memberFunc)(P1, P2, P3, P4);
	public:
		MemberConnection4(T* object, memberFunc member):_object(object), _memberFunc(member){}
		Connection4Base<R, P1, P2, P3, P4>* clone(){return new MemberConnection4(_object, _memberFunc);}
		R emit(P1 p1, P2 p2, P3 p3, P4 p4){return (_object->*_memberFunc)(p1, p2, p3, p4);}			
	private:		
		T* _object;
		memberFunc _memberFunc;	
	};

	template <typename T, typename R, typename P1, typename P2, typename P3, typename P4, typename P5>
	class MemberConnection5 : public Connection5Base<R, P1, P2, P3, P4, P5>
	{		
		typedef R (T::*memberFunc)(P1, P2, P3, P4, P5);
	public:
		MemberConnection5(T* object, memberFunc member):_object(object), _memberFunc(member){}
		Connection5Base<R, P1, P2, P3, P4, P5>* clone(){return new MemberConnection5(_object, _memberFunc);}
		R emit(P1 p1, P2 p2, P3 p3, P4 p4, P5 p5){return (_object->*_memberFunc)(p1, p2, p3, p4, p5);}			
	private:		
		T* _object;
		memberFunc _memberFunc;	
	};

	//delegate class
	template <typename R>
	class Delegate0
	{
	public:
		Delegate0():_conn(NULL){}
		template<typename Func>
		Delegate0(Func func):_conn(new Connection0<Func, R>(func))
		{
		}		
		template<typename T>
		Delegate0(T* object, R (T::*memFunc)()):_conn(new MemberConnection0<T, R>(object, memFunc))
		{
		}

		Delegate0(const Delegate0& d)
		{
			_conn = d._conn->clone();			
		}
		~Delegate0()
		{
			delete _conn;
		}
		template<typename Func>
		void bind(Func func)
		{
			Connection0<R> conn = new Connection0<R>(func);
			delete _conn;
			_conn = conn;			
		}
		template<typename T>
		void bind(T* object, R (T::*memFunc)())
		{
			MemberConnection0<T,R>* conn = new MemberConnection0<T,R>(object, memFunc);
			delete _conn;
			_conn = conn;			
		}		
		const Delegate0& operator = (const Delegate0& d)
		{
			Connection0Base<R>* conn = d._conn->clone();
			delete _conn;
			_conn = conn;
			return *this;
		}
		R operator()(){return invoke();}
		R invoke(){return _conn->emit();}
		operator bool(){return _conn != NULL;}
	private:
		Connection0Base<R>* _conn;		
	};


	template <typename R, typename P1>
	class Delegate1
	{
	public:
		Delegate1():_conn(NULL){}
		template<typename Func>
		Delegate1(Func func):_conn(new Connection1<Func, R, P1>(func))
		{
		}		
		template<typename T>
		Delegate1(T* object, R (T::*memFunc)(P1)):_conn(new MemberConnection1<T, R, P1>(object, memFunc))
		{
		}

		Delegate1(const Delegate1& d)
		{
			_conn = d._conn->clone();			
		}
		~Delegate1()
		{
			delete _conn;
		}
		template<typename Func>
		void bind(Func func)
		{
			Connection1<R, P1> conn = new Connection1<R, P1>(func);
			delete _conn;
			_conn = conn;			
		}
		template<typename T>
		void bind(T* object, R (T::*memFunc)(P1))
		{
			MemberConnection1<T, R, P1>* conn = new MemberConnection1<T, R, P1>(object, memFunc);
			delete _conn;
			_conn = conn;			
		}		
		const Delegate1& operator = (const Delegate1& d)
		{
			Connection1Base<R, P1>* conn = d._conn->clone();
			delete _conn;
			_conn = conn;
			return *this;
		}
		R operator()(P1 p1){return invoke(p1);}
		R invoke(P1 p1){return _conn->emit(p1);	}
		operator bool(){return _conn != NULL;}
	private:
		Connection1Base<R, P1>* _conn;		
	};

	template <typename R, typename P1, typename P2>
	class Delegate2
	{
	public:
		Delegate2():_conn(NULL){}
		template<typename Func>
		Delegate2(Func func):_conn(new Connection2<Func, R, P1, P2>(func))
		{
		}		
		template<typename T>
		Delegate2(T* object, R (T::*memFunc)(P1, P2)):_conn(new MemberConnection2<T, R, P1, P2>(object, memFunc))
		{
		}

		Delegate2(const Delegate2& d)
		{
			_conn = d._conn->clone();			
		}
		~Delegate2()
		{
			delete _conn;
		}
		template<typename Func>
		void bind(Func func)
		{
			Connection2<R, P1, P2> conn = new Connection2<R, P1, P2>(func);
			delete _conn;
			_conn = conn;			
		}
		template<typename T>
		void bind(T* object, R (T::*memFunc)(P1, P2))
		{
			MemberConnection2<T, R, P1, P2>* conn = new MemberConnection2<T, R, P1, P2>(object, memFunc);
			delete _conn;
			_conn = conn;			
		}		
		const Delegate2& operator = (const Delegate2& d)
		{
			Connection2Base<R, P1, P2>* conn = d._conn->clone();
			delete _conn;
			_conn = conn;
			return *this;
		}
		R operator()(P1 p1, P2 p2){return invoke(p1, p2);}
		R invoke(P1 p1, P2 p2){return _conn->emit(p1,p2);	}
		operator bool(){return _conn != NULL;}
	private:
		Connection2Base<R, P1, P2>* _conn;		
	};

	template <typename R, typename P1, typename P2, typename P3>
	class Delegate3
	{
	public:
		Delegate3():_conn(NULL){}
		template<typename Func>
		Delegate3(Func func):_conn(new Connection3<Func, R, P1, P2, P3>(func))
		{
		}		
		template<typename T>
		Delegate3(T* object, R (T::*memFunc)(P1, P2, P3)):_conn(new MemberConnection3<T, R, P1, P2, P3>(object, memFunc))
		{
		}

		Delegate3(const Delegate3& d)
		{
			_conn = d._conn->clone();			
		}
		~Delegate3()
		{
			delete _conn;
		}
		template<typename Func>
		void bind(Func func)
		{
			Connection3<R, P1, P2, P3> conn = new Connection3<R, P1, P2, P3>(func);
			delete _conn;
			_conn = conn;			
		}
		template<typename T>
		void bind(T* object, R (T::*memFunc)(P1, P2, P3))
		{
			MemberConnection3<T, R, P1, P2, P3>* conn = new MemberConnection3<T, R, P1, P2, P3>(object, memFunc);
			delete _conn;
			_conn = conn;			
		}		
		const Delegate3& operator = (const Delegate3& d)
		{
			Connection3Base<R, P1, P2, P3>* conn = d._conn->clone();
			delete _conn;
			_conn = conn;
			return *this;
		}
		R operator()(P1 p1, P2 p2, P3 p3){return invoke(p1, p2, p3);}
		R invoke(P1 p1, P2 p2, P3 p3){return _conn->emit(p1,p2,p3);	}
		operator bool(){return _conn != NULL;}
	private:
		Connection3Base<R, P1, P2, P3>* _conn;		
	};

	//signal classes:
	template <typename ThreadModel>
	class Signal0
	{			
	public:
		Signal0(){}
		~Signal0()
		{
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				delete (*it);				
			}		
		}
		template <typename FUNC>
		conn_t connect(FUNC func)
		{
			SignalLock lock(_lock);			
			_connections.push_back(new Connection0<FUNC, void>(func));			
			return _connections.back();
		}

		template <typename T>
		conn_t connect(T* object, void (T::*memFunc)())
		{
			SignalLock lock(_lock);					
			_connections.push_back(new MemberConnection0<T, void>(object, memFunc));			
			return _connections.back();
		}		
		void disconnect(conn_t conn)
		{			
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				if((*it) == conn)
				{					
					delete (*it); _connections.erase(it);				
					return;
				}
			}			
			throw ObjectExists(_T("Connection not found"));
		}

		void operator()()
		{
			emit();
		}
		void emit()
		{
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				(*it)->emit();
			}			
		}
	private:
		typedef std::vector<Connection0Base<void>* >CONNECTIONS;
		CONNECTIONS _connections;		
		typedef ScopeLock<typename ThreadModel::LockType> SignalLock;
		typename ThreadModel::LockType _lock;	
	};

	template <typename P1, typename ThreadModel>
	class Signal1
	{			
	public:
		Signal1(){}
		~Signal1()
		{
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				delete (*it);				
			}		
		}
		template <typename FUNC>
		conn_t connect(FUNC func)
		{
			SignalLock lock(_lock);			
			_connections.push_back(new Connection1<FUNC,void,P1>(func));			
			return _connections.back();
		}

		template <typename T>
		conn_t connect(T* object, void (T::*memFunc)(P1))
		{
			SignalLock lock(_lock);					
			_connections.push_back(new MemberConnection1<T, void, P1>(object, memFunc));			
			return _connections.back();
		}		
		void disconnect(conn_t conn)
		{			
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				if((*it) == conn)
				{
					delete (*it); _connections.erase(it);				
					return;
				}
			}			
			throw ObjectExists(_T("Connection not found"));
		}

		void operator()(P1 p1)
		{
			emit(p1);
		}
		void emit(P1 p1)
		{
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				(*it)->emit(p1);
			}			
		}
	private:
		typedef std::vector<Connection1Base<void, P1>*> CONNECTIONS;
		CONNECTIONS _connections;		
		typedef ScopeLock<typename ThreadModel::LockType> SignalLock;
		typename ThreadModel::LockType _lock;	
	};

	template <typename P1, typename P2, typename ThreadModel>
	class Signal2
	{			
	public:
		Signal2(){}
		~Signal2()
		{
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				delete (*it);				
			}		
		}
		template <typename FUNC>
		conn_t connect(FUNC func)
		{
			SignalLock lock(_lock);			
			_connections.push_back(new Connection2<FUNC, void , P1,P2>(func));			
			return _connections.back();
		}

		template <typename T>
		conn_t connect(T* object, void (T::*memFunc)(P1, P2))
		{
			SignalLock lock(_lock);					
			_connections.push_back(new MemberConnection2<T, void, P1, P2>(object, memFunc));			
			return _connections.back();
		}		
		void disconnect(conn_t conn)
		{			
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				if((*it) == conn)
				{					
					delete (*it); _connections.erase(it);				
					return;
				}
			}			
			throw ObjectExists(_T("Connection not found"));
		}

		void operator()(P1 p1, P2 p2)
		{
			emit(p1, p2);
		}
		void emit(P1 p1, P2 p2)
		{
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				(*it)->emit(p1,p2);
			}			
		}
	private:
		typedef std::vector<Connection2Base<void, P1, P2>*> CONNECTIONS;
		CONNECTIONS _connections;		
		typedef ScopeLock<typename ThreadModel::LockType> SignalLock;
		typename ThreadModel::LockType _lock;	
	};

	template <typename P1, typename P2, typename P3, typename ThreadModel>
	class Signal3
	{			
	public:
		Signal3(){}
		~Signal3()
		{
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				delete (*it);				
			}		
		}
		template <typename FUNC>
		conn_t connect(FUNC func)
		{
			SignalLock lock(_lock);			
			_connections.push_back(new Connection3<FUNC,void,P1,P2,P3>(func));			
			return _connections.back();
		}

		template <typename T>
		conn_t connect(T* object, void (T::*memFunc)(P1, P2, P3))
		{
			SignalLock lock(_lock);					
			_connections.push_back(new MemberConnection3<T, void, P1, P2, P3>(object, memFunc));			
			return _connections.back();
		}		
		void disconnect(conn_t conn)
		{			
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				if((*it) == conn)
				{
					delete (*it); _connections.erase(it);				
					return;
				}
			}			
			throw ObjectExists(_T("Connection not found"));
		}

		void operator()(P1 p1, P2 p2, P3 p3)
		{
			emit(p1, p2, p3);
		}
		void emit(P1 p1, P2 p2, P3 p3)
		{
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				(*it)->emit(p1,p2,p3);
			}			
		}
	private:
		typedef std::vector<Connection3Base<void, P1, P2, P3>*> CONNECTIONS;
		CONNECTIONS _connections;		
		typedef ScopeLock<typename ThreadModel::LockType> SignalLock;
		typename ThreadModel::LockType _lock;	
	};

	template <typename P1, typename P2, typename P3, typename P4, typename ThreadModel>
	class Signal4
	{			
	public:
		Signal4(){}
		~Signal4()
		{
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				delete (*it);				
			}		
		}
		template <typename FUNC>
		conn_t connect(FUNC func)
		{
			SignalLock lock(_lock);			
			_connections.push_back(new Connection4<FUNC,void, P1,P2,P3,P4>(func));			
			return _connections.back();
		}

		template <typename T>
		conn_t connect(T* object, void (T::*memFunc)(P1, P2, P3, P4))
		{
			SignalLock lock(_lock);					
			_connections.push_back(new MemberConnection4<T, void, P1, P2, P3, P4>(object, memFunc));			
			return _connections.back();
		}		
		void disconnect(conn_t conn)
		{			
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				if((*it) == conn)
				{
					delete (*it); _connections.erase(it);				
					return;
				}
			}			
			throw ObjectExists(_T("Connection not found"));
		}

		void operator()(P1 p1, P2 p2, P3 p3, P4 p4)
		{
			emit(p1, p2, p3, p4);
		}
		void emit(P1 p1, P2 p2, P3 p3, P4 p4)
		{
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				(*it)->emit(p1,p2,p3,p4);
			}			
		}
	private:
		typedef std::vector<Connection4Base<void, P1, P2, P3, P4>*> CONNECTIONS;
		CONNECTIONS _connections;		
		typedef ScopeLock<typename ThreadModel::LockType> SignalLock;
		typename ThreadModel::LockType _lock;	
	};

	template <typename P1, typename P2, typename P3, typename P4, typename P5, typename ThreadModel>
	class Signal5
	{			
	public:
		Signal5(){}
		~Signal5()
		{
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				delete (*it);				
			}		
		}
		template <typename FUNC>
		conn_t connect(FUNC func)
		{
			SignalLock lock(_lock);			
			_connections.push_back(new Connection5<FUNC, void, P1,P2,P3,P4,P5>(func));			
			return _connections.back();
		}

		template <typename T>
		conn_t connect(T* object, void (T::*memFunc)(P1, P2, P3, P4, P5))
		{
			SignalLock lock(_lock);					
			_connections.push_back(new MemberConnection5<T, void, P1, P2, P3, P4, P5>(object, memFunc));			
			return _connections.back();
		}		
		void disconnect(conn_t conn)
		{			
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				if((*it) == conn)
				{
					delete (*it); _connections.erase(it);				
					return;
				}
			}			
			throw ObjectExists(_T("Connection not found"));
		}

		void operator()(P1 p1, P2 p2, P3 p3, P4 p4, P5 p5)
		{
			emit(p1, p2, p3, p4, p5);
		}
		void emit(P1 p1, P2 p2, P3 p3, P4 p4, P5 p5)
		{
			SignalLock lock(_lock);
			for(CONNECTIONS::iterator it = _connections.begin(); it != _connections.end(); ++it)
			{
				(*it)->emit(p1,p2,p3,p4,p5);
			}			
		}
	private:
		typedef std::vector<Connection5Base<void, P1, P2, P3, P4, P5>*> CONNECTIONS;
		CONNECTIONS _connections;		
		typedef ScopeLock<typename ThreadModel::LockType> SignalLock;
		typename ThreadModel::LockType _lock;	
	};
}//namespace

#endif //_SIGNALH_

