#pragma once

#include <shlwapi.h>                    // for StrFormatByteSize()
#include "gnthread.h"
#include "GNUtil.h"
#include "resource.h"

class CDownloadStatusCallBack : 
	public IBindStatusCallback,
	public CGNThread
{
public:
#define STRINGBUF_SIZE 256

	CDownloadStatusCallBack(void) :
		  CGNThread(false), // don't self destruct
		  m_bAbortDownload(false), m_hwndToNotify(NULL), m_nRefCount(0)
	  {
	  }

	  virtual ~CDownloadStatusCallBack(void)
	  {
	  }

	  virtual DWORD Run()
	  {
		  AddRef();

	  	  CString fileName;
		  int pos = m_url.ReverseFind(_T('/'));
		  if(pos <= 0)
		  {
			  m_IEStatusMsg = _T("Invalid url");
			  ::PostMessage(m_hwndToNotify, WM_COMMAND, ID_DOWNLOAD_FAILED, 0);
			  return 0;
		  }
		  fileName = m_url.Mid(pos+1);

		  CString temp = CGNUtil::GetTempPathName()+fileName;
		  _tcscpy(m_downloadedFile, (LPCTSTR)temp);
		  HRESULT hr = ::URLDownloadToFile(NULL, 
			  m_url,
			  m_downloadedFile,
			  0,
			  (IBindStatusCallback*)this);

		  if(hr == S_OK)
		  {
			  ::PostMessage(m_hwndToNotify, WM_COMMAND, ID_DOWNLOAD_READY, 0);
		  }
		  else
		  {
		      CComCritSecLock<CComAutoCriticalSection> lock(m_lock);

			  LPTSTR lpszErrorMessage;
			  if (FormatMessage ( FORMAT_MESSAGE_ALLOCATE_BUFFER | 
									FORMAT_MESSAGE_FROM_SYSTEM | 
									FORMAT_MESSAGE_IGNORE_INSERTS,
									NULL, hr, 
									MAKELANGID ( LANG_NEUTRAL, SUBLANG_DEFAULT ),
									(LPTSTR) &lpszErrorMessage, 0, NULL ))
			  {
				  m_IEStatusMsg.Format ( _T("Download failed.  Error = 0x%08lX\n\n%s"),
					  (DWORD) hr, lpszErrorMessage );

				  LocalFree ( lpszErrorMessage );
			  }
			  else
			  {
				  m_IEStatusMsg.Format ( _T("Download failed.  Error = 0x%08lX\n\nNo message available."),
					  (DWORD) hr );
			  }
			  
			  lock.Unlock();

			  ::PostMessage(m_hwndToNotify, WM_COMMAND, ID_DOWNLOAD_FAILED, 0);
		  }

		  return 0;
	  }


public:
	STDMETHOD(OnProgress)(/* [in] */ ULONG ulProgress,
							/* [in] */ ULONG ulProgressMax,
							/* [in] */ ULONG ulStatusCode,
							/* [in] */ LPCWSTR wszStatusText)
	{
		// Did the user hit the Stop button?
		if ( m_bAbortDownload )
			return E_ABORT;

		CComCritSecLock<CComAutoCriticalSection> lock(m_lock);

		// Use CString to convert IE's status message to a TCHAR string.
		if ( NULL != wszStatusText )
		{
			m_IEStatusMsg = wszStatusText;
		}
		else
		{
			m_IEStatusMsg.Empty();
		}

		// Make our own progress message - we'll show the amount downloaded and
		// the total file size (if known).

		StrFormatByteSize ( ulProgress, m_szAmtDownloaded, STRINGBUF_SIZE );
		StrFormatByteSize ( ulProgressMax, m_szTotalSize, STRINGBUF_SIZE );

		if ( 0 != ulProgressMax )
		{
			wsprintf ( m_szCustomStatusMsg, _T("Downloaded %s of %s"),
				m_szAmtDownloaded, m_szTotalSize );
		}
		else
		{
			wsprintf ( m_szCustomStatusMsg, _T("Downloaded %s (total size unknown)"),
				m_szAmtDownloaded );
		}

		m_progress = int( 100.0 * ulProgress / ulProgressMax);

		lock.Unlock();

		// Report the progress back to the main window.
		::PostMessage(m_hwndToNotify, WM_COMMAND, ID_DOWNLOAD_PROGRESS, 0);

		return S_OK;
	}


	STDMETHODIMP OnStartBinding(DWORD, IBinding *)
	{
		return E_NOTIMPL;
	}
	STDMETHODIMP GetPriority(LONG *)
	{
		return E_NOTIMPL;
	}
	STDMETHODIMP OnLowResource(DWORD)
	{
		return S_OK;
	}
	STDMETHODIMP OnStopBinding(HRESULT, LPCWSTR)
	{
		return S_OK;
	}
	STDMETHODIMP GetBindInfo(DWORD *, BINDINFO *)
	{
		return S_OK;
	}
	STDMETHODIMP OnDataAvailable(DWORD, DWORD, FORMATETC *, STGMEDIUM *)
	{
		return S_OK;
	}
	STDMETHODIMP OnObjectAvailable(REFIID, IUnknown *)
	{
		return S_OK;
	}

public:

	//
	//	IUnknown implementation
	//
	STDMETHODIMP_(ULONG) AddRef()
	{
		return ::InterlockedIncrement(reinterpret_cast<LONG*>(&m_nRefCount));
	}

	STDMETHODIMP_(ULONG) Release()
	{
		ULONG nCount = 0;
		nCount = (ULONG) ::InterlockedDecrement(reinterpret_cast<LONG*>(&m_nRefCount));

		if (nCount == 0)
			delete this;

		return nCount;

	}

	STDMETHODIMP QueryInterface(REFIID riid, void** ppvObject)
	{

		HRESULT hr = E_NOINTERFACE;
		
		if (ppvObject != NULL)
		{
			*ppvObject = NULL;

			if (IID_IUnknown == riid)
				*ppvObject = static_cast<IUnknown*>(this);

			if (IID_IBindStatusCallback == riid)
				*ppvObject = static_cast<IBindStatusCallback*>(this);

			if (*ppvObject != NULL)
			{
				hr = S_OK;
				((LPUNKNOWN)*ppvObject)->AddRef();
			}

		}
		else
		{
			hr = E_POINTER;
		}
		
		return hr;

	}


public:
	// out
	CComAutoCriticalSection m_lock;
	CString m_IEStatusMsg;
	TCHAR m_szCustomStatusMsg [STRINGBUF_SIZE];
	TCHAR m_szAmtDownloaded [STRINGBUF_SIZE], m_szTotalSize [STRINGBUF_SIZE];
	TCHAR m_downloadedFile[MAX_PATH];
	int m_progress;

	// in
	bool m_bAbortDownload;
	HWND m_hwndToNotify;
	CString m_url;

private:
	int m_nRefCount;
};
